#!/usr/bin/env python

# omg brumby Rel 3
# BRL to visible layers requested by Wallace on gimplearn.
# Created by Tin Tran http://bakon.ca/gimplearn/
# Comments directed to http://gimplearn.com or http://gimp-forum.net or http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Requested by Pat to keep names of layers.
# Rel 3: Fix bug for when there is an spherical emap image defined.
import math
import string
#import Image
from gimpfu import *
from array import array
import sys


def python_BRL_to_visible_layers(image, layer, bevel_slope, flat_bevel_smoothing, flat_bevel_depth, round1_bevel_depth, round2_bevel_depth, diffuse, contour, contour_depth, edge_smoothing_amount, spherical_emap_png, env_map_type, blending_mode, blend_direction, blend_shape, select_gradient, light_direction, emap_spec, detail_level, horizontal_texture_size,vertical_texture_size,emap_final_size,emap_final_blur,auto_stretch_HSV,turbulent_noise,keep_layers):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#copy visible layes into array first so that we don't mix it up with newly generated layers
	layers = []
	for layer in image.layers:
		if layer.visible:
			layers.append(layer)
	
	#now loop through and perform BRL.
	for layer in layers:
		name = layer.name
		pdb.script_fu_ml_bevel_reflect_logo_alpha(image,layer,bevel_slope, flat_bevel_smoothing, flat_bevel_depth, round1_bevel_depth, round2_bevel_depth, diffuse, contour, contour_depth, edge_smoothing_amount, spherical_emap_png, env_map_type, blending_mode, blend_direction, blend_shape, select_gradient, light_direction, emap_spec, detail_level, horizontal_texture_size,vertical_texture_size,emap_final_size,emap_final_blur,auto_stretch_HSV,turbulent_noise,keep_layers)
		#hack to keep same original names of layers
		result = pdb.gimp_image_get_layer_by_name(image,"BR Logo")
		result.name = name
		spherical_topo = pdb.gimp_image_get_layer_by_name(image,"Spherical Topo")
		if spherical_topo:
			spherical_topo.name = name + " Spherical Topo"
		if keep_layers:
			copy = pdb.gimp_image_get_layer_by_name(image,"BR Logo Copy")
			copy.name = name + " BR Logo Copy"	
	
	
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_BRL_to_visible_layers",                           
	"BRL to visible layers",
	"BRL to visible layers",
	"Tin Tran",
	"Tin Tran",
	"2017",
	"<Image>/Python-Fu/BRL to visible layers...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_OPTION,"bevel_slope",   "Bevel - Slope Options:", 0, ["Flat Bevel","Round1 Bevel","Round2 Bevel","Run Interactive"]),
	(PF_SPINNER, "flat_bevel_smoothing", "Flat Bevel - Smoothing:", 8, (1, 32, 1)),
	(PF_SPINNER, "flat_bevel_depth", "Flat Bevel - Depth:", 10.0, (1.0, 10.5, 0.1)),
	(PF_SPINNER, "round1_bevel_depth", "Round1 Bevel - Depth:", 7.0, (1.0, 8.0, 0.1)),
	(PF_SPINNER, "round2_bevel_depth", "Round2 Bevel - Depth:", 20, (1, 25, .1)),
	(PF_SPINNER, "diffuse", "Diffuse 0<-->1 (Shiny):", 1, (0, 1, 0.1)),
	(PF_OPTION,"contour",   "Contour:", 0, ["Linear", "Outline", "Outline (Inner Bevel)", "Outline (Outer Bevel)", "Step Bevel", "Ring (Narrow)", "Ring (Medium)", "Ring (Wide)", "Ring (Extra Wide)", "Ring (Thick)", "Double Ring", "Double Ring (Narrow)", "Double Ring (Wide)"]),
	(PF_SPINNER, "contour_depth", "Contour Depth:", 0, (0, 20, 1)),
	(PF_SPINNER, "edge_smoothing_amount", "Edge Smoothing Amount:", 1, (0, 4, 1)),
	(PF_FILENAME, "spherical_emap_png", "Spherical Emap (png/jpg):", "none"),
	(PF_OPTION,"env_map_type",   "Environment Map Type:", 0, ["Spherical Topographical", "Spherical Cloud", "Spherical Topo (Active Gradient)", "Spherical Cloud (Active Gradient)", "Spherical Solid (Active Color)", "Generate Normal Map"]),
	(PF_OPTION,"blending_mode",   "Blending Mode:", 0, ["Normal", "Dissolve", "Behind", "Multiply", "Screen", "Overlay", "Difference", "Addition", "Subtract", "Darken", "Lighten", "Hue", "Saturation" ,"Color", "Value", "Divide", "Dodge", "Burn", "Hard Light", "Soft Light", "Grain Extract", "Grain Merge"]),
	(PF_OPTION,"blend_direction",   "Blend Direction:", 0, ["Top-Botton", "Bottom-Top", "Left-Right", "Right-Left" ,"Diag-Top-Left", "Diag-Top-Right", "Diag-Bottom-Left", "Diag-Bottom-Right"]),
	(PF_OPTION,"blend_shape",   "Blend Shape:", 0, ["Linear", "Bilinear", "Radial", "Square", "Conical-Symmetric", "Conical-Asymmetric", "Shapeburst-Angular", "Shapburst-Spherical", "Shapeburst-Dimpled", "Spiral-CW", "Spiral-CCW"]),
	(PF_GRADIENT, "select_gradient", "Select Gradient:", "Abstract 1"),
	(PF_SPINNER, "light_direction", "Light Direction (Degrees):", 315, (0, 360, 1)),
	(PF_SPINNER, "emap_spec", "Emap Specular Highlights:", 0.35, (0.10, 0.65, 0.01)),
	(PF_SPINNER, "detail_level", "Detail Level:", 0, (0, 3, 1)),
	(PF_SPINNER, "horizontal_texture_size", "Horizontal Texture Size:", 1.2, (1, 5, 0.1)),
	(PF_SPINNER, "vertical_texture_size", "Vertical Texture Size:", 2.6, (1, 5, 0.1)),
	(PF_SPINNER, "emap_final_size", "Emap Final Size:", 300, (250, 1000, 1)),
	(PF_SPINNER, "emap_final_blur", "Emap Final Blur:", 0, (0, 20, 1)),
	(PF_TOGGLE, "auto_stretch_HSV",   "Autostretch HSV:", 1), # initially True, checked.
	(PF_TOGGLE, "turbulent_noise",   "Turbulent Noise:", 1), # initially True, checked.
	(PF_TOGGLE, "keep_layers",   "Keep Layers:", 1), # initially True, checked.
	],
	[],
	python_BRL_to_visible_layers)

main()
