#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

This plugins is a combination of two plug-ins Adrian Likins ( http://www.adrianlikins.com/misc/gimp/ ): font-to-brushes.py and font-to-image-hose.py

Attention: Do not use these characters \/?:*.><| to save the as file
'''

import os, re
import string

import gimp
from gimpfu import *
import pygtk
pygtk.require("2.0")
import gtk


def messBox(message, type, modal):
    if modal == 0:
      flag = gtk.DIALOG_DESTROY_WITH_PARENT
    else:
      flag = gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT
    msgBox = gtk.MessageDialog(None, flag, type, gtk.BUTTONS_OK, message)
    ret = msgBox.run()
    msgBox.destroy()

def get_gimp_version():
    # assume version like "gimp-2.8"
    ver = pdb.gimp_version()
    parts = ver.split(".")
    #    print parts
    major_minor_ver = string.join(parts[0:2], ".")
    return major_minor_ver
    
def add_fontname_to_all_layers(img, name): 
	if name:
		for layer in img.layers:
			layer.name += name
	else:
		number = 0
		for layer in img.layers:
			layer.name = str(number)
			number += 1
			
def font_to_layers(font, chars, size):
    print font, chars, size
    chars = chars.decode("utf-8")
    parts = font.split(" ")
    if parts[-1].isdigit():
        fontparts  = parts[:-1]
        font = string.join(fontparts, " ")

    max_width = 0.0
    max_height = 0.0
    for char in chars:
        (width, height, asc, dsc) = pdb.gimp_text_get_extents_fontname(char, size, 1, font)
        if width > max_width:
            max_width = width
        if height > max_height:
            max_height = height

    img = gimp.Image(max_width, max_height, RGB)

    layer_count = 0
    for char in chars:
        (width, height, asc, dsc) = pdb.gimp_text_get_extents_fontname(char, size, 1, font)

        drawable = gimp.Layer(img, "%s" % char, width, height, RGBA_IMAGE, 100, NORMAL_MODE)
        img.add_layer(drawable, layer_count)
        layer_count = layer_count + 1
        pdb.gimp_edit_clear(drawable)
        font_layer = pdb.gimp_text_fontname(img, drawable,  0.0, 0.0, char, 0, 1, size, 1, font)

        pdb.gimp_floating_sel_anchor(font_layer)

    name = "%s.png%s"  % ("","")
    filename = "%s/%s.png" % ("",name)

    print img, drawable, filename, filename, 50, name, max_width, max_height, 1, 1, 1, [len(chars)], 1
    disp = gimp.Display(img)
    pdb.python_fu_add_fontname_to_all_layers( img, name, run_mode=RUN_INTERACTIVE)
    gimp_copy = drawable.copy(0)
    img.add_layer(gimp_copy, 0)
    pdb.gimp_edit_clear(gimp_copy)
    gimp_copy.name = "space.png"	

    messBox("""Attention:
    Do not use these characters \ / ? : * . > < | 
    to save (export) layers as file.
    This plugins is starter for Custom font.scm by Tin Tran.
    (therefore adds space.png).
    Add Your effect for all layers and save as *.xcf
    Hint:
    For effects on all layers can use GMIC or GAP (Filter all Layers)
    """, gtk.MESSAGE_INFO, 1)    
    return
	
register(
    "python_fu_font_to_layers",
    "Opens letters of the selected font as layers.",
    "Font to layers...",
    "MareroQ",
    "GPL",
    "2016",
    "<Toolbox>/Filters/Text/Font to layers...",
    "",
    [
    (PF_FONT, "font", "Font: ", "Sans"),
    (PF_STRING, "text", "Characters: ", "abcdefghijklmnopqrstuvxyzABCDEFGHIJKLMNOPQRSTUVXYZ0123456789\/?:*.><|~!@#$%^&()_+-=;',{}[]"),
    (PF_FLOAT, "size", "Size of Font: ", 20.0),

    ],
    [],
    font_to_layers)

register(
    proc_name=("python-fu-add-fontname-to-all-layers"),
    blurb=(""),
    help=("Help?"),
    author=(""),
    copyright=("GPL"),
    date=("2016"),
    label=(""),
    imagetypes=("*"),
    params=[
	(PF_IMAGE, "img", "Image", None),
	(PF_STRING, "name", "Add text:", ""),
	],
    results=[],
    function=(add_fontname_to_all_layers), 
    menu=("")
 )
 
main()