#!/usr/bin/env python

# Guides_In_Range_px Rel 1
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
#        Plugin based on 'Add_Guides_In_Range.py' by Tin Tran https://gimplearn.net/viewtopic.php?f=3&t=6
# Rel 2: add limit to x/y END
#        added "Draw Guides Pencil" (by active brush, FG color, on new layer) option 
#		 added "Delete all guides" option

import math
import string
from gimpfu import *
from array import array


def python_add_guides_in_range_px(image, layer, remove_all, orientation_x, range_start_x, range_end_x, sections_x, orientation_y, range_start_y, range_end_y, sections_y, draw_guides) :
	
	pdb.gimp_image_undo_group_start(image)
	
	width = pdb.gimp_image_width(image)
	height = pdb.gimp_image_height(image)
	
	if draw_guides==1:
		pdb.gimp_context_set_paint_method("gimp-pencil")
		new_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,'Guide',100,LAYER_MODE_NORMAL_LEGACY)
		pdb.gimp_image_insert_layer(image,new_layer,None,0)
		
	if remove_all:
		guideId = pdb.gimp_image_find_next_guide(image, 0)
		while guideId <> 0:
			pdb.gimp_image_delete_guide(image, guideId)
			guideId = pdb.gimp_image_find_next_guide(image, 0)	
	
	if range_end_x>height:
		range_end_x=height
	if range_end_y>width:
		range_end_y=width		
		
	if orientation_x: #vertical guides
	
		span_width = (range_end_x - range_start_x)/sections_x #calculate the span width we're dealing with
		for i in range(0,int(sections_x+1)):
			guide_position_x = range_start_x + i * (span_width)
			#pdb.script_fu_guide_new(image,layer,0,guide_position)
			pdb.gimp_image_add_hguide(image, guide_position_x)
			if draw_guides==1:
				pdb.gimp_pencil(new_layer, 4, (0, guide_position_x, width, guide_position_x))
	if orientation_y:                #horizontal guides

		span_height = (range_end_y - range_start_y)/sections_y		
		for i in range(0,int(sections_y+1)):
			guide_position_y = range_start_y + i * (span_height)
			#pdb.script_fu_guide_new(image,layer,1,guide_position)
			pdb.gimp_image_add_vguide(image, guide_position_y)			
			if draw_guides==1:
				pdb.gimp_pencil(new_layer, 4, (guide_position_y, 0, guide_position_y, height))
		
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_fu_add_guides_in_range_px",
	"Add Multiple Guides in Range",
	"Add Multiple Guides in Range...",
	"MrQ",
	"GPL",
	"2022",
	"<Image>/Image/Guides/Guides in Range (px)...",
	"*",      # Create a new image, don't work on an existing one
	[
	(PF_RADIO,   "remove_all", "Remove all Guides:", 0, (("Yes", 1),("No",0))),	
	(PF_RADIO,   "orientation_x", "Orientation Horizontal:", 0, (("Yes", 1),("No",0))),
	(PF_SPINNER, "range_start", "Y1 Range Start (px):", 0, (0, 5000, 1)),
	(PF_SPINNER, "range_end", "Y2 Range End (px):\n(max=image height)", 1, (1, 5000, 1)),
	(PF_SPINNER, "sections", "Number of Rows:", 1, (1, 1000, 1)),
	(PF_RADIO,   "orientation_y", "Orientation Vertical:", 0, (("Yes", 1),("No",0))),	
	(PF_SPINNER, "range_start", "X1 Range Start (px):", 0, (0, 5000, 1)),
	(PF_SPINNER, "range_end", "X2 Range End (px):\n(max=image width)", 1, (1, 5000, 1)),
	(PF_SPINNER, "sections", "Number of Columns:", 1, (1, 1000, 1)),
	(PF_RADIO,   "draw_guides", "Draw Guides Pencil:\n(active brush, FG color)", 0, (("Yes", 1),("No",0))),	
	],
	[],
	python_add_guides_in_range_px)

main()
