#!/usr/bin/env python

# Pat625_Mandala_With_Your_Name Rel 1
# Created by Tin Tran http://bakon.ca/gimplearn/
# This script follows tutorial described at http://www.mahvin.com/2010/02/gimp-tutorial-camouflage-revisited.html
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# This tutorial produces results like shown in tutorial shown at http://bakon.ca/gimplearn/viewtopic.php?f=10&t=268
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.


import math
import string
#import Image
from gimpfu import *
from array import array
import random
#===== methods to be used when we want to stroke a path. Tin Tran did not write vector_to_line_stroke, I found it on the net somewhere 
# and made minor changes to it.
def color_to_hex(color):
	whole_color = (color[0],color[1],color[2]);
	return '#%02x%02x%02x' % whole_color
def vector_to_line_stroke(image, vector, layer, color="#000000", width=1, capstyle="butt", joinstyle="miter", miterlimit=10, shaperendering="auto"):
    import re, tempfile
    import os, string, sys
    newelements = {
            'stroke': color,
            'stroke-width': width,
            'stroke-linecap': capstyle,
            'stroke-linejoin': joinstyle,
            'stroke-miterlimit': miterlimit,
			'shape-rendering': shaperendering,
            }
    svg = pdb.gimp_vectors_export_to_string(image, vector)
    #fix width and height to be resolution (px/inch)-independent
    svg = re.sub(r'(<svg\s[^>]*\swidth\s*=\s*)\S*"', r'\1"%dpx"' % image.width, svg, flags=re.DOTALL)
    svg = re.sub(r'(<svg\s[^>]*\sheight\s*=\s*)\S*"', r'\1"%dpx"' % image.height, svg, flags=re.DOTALL)
    svg = re.sub(r'(<path\s[^>]*)\sstroke\s*=\s*"black"', r'\1', svg, flags=re.DOTALL)
    svg = re.sub(r'(<path\s[^>]*)\sstroke-width\s*=\s*"1"', r'\1', svg, flags=re.DOTALL)
    svg = re.sub(r'(<path\s)', r'\1' + ''.join([r'%s="%s" ' % i for i in newelements.items()]), svg, flags=re.DOTALL)
    tmpfile = tempfile.TemporaryFile(suffix=".svg")
	
    #gets just the file name to use, added by Tin Tran, tempFile.TemporaryFile above gave write permission error
    filename = os.path.basename(tmpfile.name)
#    svgfile =  "~\\.gimp-2.8\\tmp\\" + filename
    svgfile =  "~\\" + filename
    svgfile = os.path.expanduser(svgfile)
    tmpfile = open(svgfile,"w")
	#end of addition by Tin Tran
	
    tmpfile.write(svg)
    tmpfile.flush();
    newlayer = pdb.gimp_file_load_layer(image, tmpfile.name)
    tmpfile.close()
    image.add_layer(newlayer) #needs to be added to the image to be able to copy from
    copyname = pdb.gimp_edit_named_copy(newlayer, "stroke")
    image.remove_layer(newlayer)
    floating_sel = pdb.gimp_edit_named_paste(layer, copyname, True)
    pdb.gimp_floating_sel_anchor(floating_sel)
	
    #added by Tin Tran, remove the file when we're done
    os.remove(svgfile)
#=====================================================================================================================

def python_tt_pat_mandala(image, layer, font, yourname, gradient, reversegradient):
	#pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#creates new image and attach it to a display
	new_image = pdb.gimp_image_new(1000,1000,RGB)
	new_display = pdb.gimp_display_new(new_image)
	
	#create background layer
	background = pdb.gimp_layer_new(new_image,1000,1000,RGBA_IMAGE,"white background",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(new_image,background,None,-1)
	pdb.gimp_context_set_foreground((255,255,255))
	pdb.gimp_drawable_fill(background,FOREGROUND_FILL)
	
	pdb.gimp_context_set_foreground((0,0,0))
	pdb.gimp_context_set_background((255,255,255))
	
	text = pdb.gimp_text_layer_new(new_image,yourname,font,100,0)
	pdb.gimp_image_insert_layer(new_image,text,None,-1)
	
	#move to position
	pdb.gimp_layer_set_offsets(text,500-text.width/2.0,500)
	pdb.gimp_layer_resize_to_image_size(text)
	
	#copy layer
	textcopy = pdb.gimp_layer_new_from_drawable(text,new_image)
	pdb.gimp_image_insert_layer(new_image,textcopy,None,-1)
	#rotate
	pdb.gimp_item_transform_rotate(textcopy,math.pi,1,500,500)
	#merge down
	text = pdb.gimp_image_merge_down(new_image,textcopy,CLIP_TO_BOTTOM_LAYER)
	
	#copy layer
	textcopy = pdb.gimp_layer_new_from_drawable(text,new_image)
	pdb.gimp_image_insert_layer(new_image,textcopy,None,-1)
	#rotate
	pdb.gimp_item_transform_rotate(textcopy,math.pi/2.0,1,500,500)
	#merge down
	text = pdb.gimp_image_merge_down(new_image,textcopy,CLIP_TO_BOTTOM_LAYER)
	
	#copy layer
	textcopy = pdb.gimp_layer_new_from_drawable(text,new_image)
	pdb.gimp_image_insert_layer(new_image,textcopy,None,-1)
	#rotate
	pdb.gimp_item_transform_rotate(textcopy,math.pi/4.0,1,500,500)
	#merge down
	text = pdb.gimp_image_merge_down(new_image,textcopy,CLIP_TO_BOTTOM_LAYER)
	
	#autocrop layer to grab width and height then resize it back
	pdb.plug_in_autocrop_layer(new_image,text)
	width = text.width
	height = text.height
	pdb.gimp_layer_resize_to_image_size(text)
	
	#large circle
	grow = 18
	pdb.gimp_image_select_ellipse(new_image,CHANNEL_OP_REPLACE,500-width/2.0-grow,500-height/2.0-grow,width+grow*2,height+grow*2)
	pdb.plug_in_sel2path(new_image,text)
	large_circle_vectors = pdb.gimp_image_get_active_vectors(new_image)
	
	pdb.gimp_selection_none(new_image)
	vector_to_line_stroke(new_image,large_circle_vectors,text,color_to_hex((0,0,0)),12,"square","bevel",10,"auto")
	
	#small circle
	grow = 0
	pdb.gimp_image_select_ellipse(new_image,CHANNEL_OP_REPLACE,500-width/2.0-grow,500-height/2.0-grow,width+grow*2,height+grow*2)
	pdb.plug_in_sel2path(new_image,text)
	small_circle_vectors = pdb.gimp_image_get_active_vectors(new_image)
	
	pdb.gimp_selection_none(new_image)
	vector_to_line_stroke(new_image,small_circle_vectors,text,color_to_hex((0,0,0)),4,"square","bevel",10,"auto")
	
	#create gradient layer
	gradient_layer = pdb.gimp_layer_new(new_image,1000,1000,RGBA_IMAGE,"gradient",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(new_image,gradient_layer,None,-1)
	pdb.gimp_image_select_item(new_image,CHANNEL_OP_REPLACE,text)
	pdb.gimp_context_set_gradient(gradient) #sets context gradient as edit blend uses it.
	pdb.gimp_edit_blend(gradient_layer,CUSTOM_MODE,NORMAL_MODE,GRADIENT_RADIAL,100,0,REPEAT_NONE,reversegradient,0,3,0.2,1,500,500,500+width/2.0+12,500)
	
	#saves it to path and stroke it
	pdb.plug_in_sel2path(new_image,text)
	mandala_vectors = pdb.gimp_image_get_active_vectors(new_image)
	pdb.gimp_selection_none(new_image)
	vector_to_line_stroke(new_image,mandala_vectors,gradient_layer,color_to_hex((0,0,0)),1,"square","bevel",10,"auto")
	
	
	#if flattenimage==1:
	#	pdb.gimp_image_flatten(new_image)
	
	
	pdb.gimp_context_pop()
	#pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_path_mandala",                           
	"Creates Mandala using your name",
	"Creates Mandala using your name...",
	"Tin Tran",
	"Tin Tran",
	"2016",
	"<Image>/Python-Fu/Create New/Pat625 Mandala With Your Name...",             #Menu path
	"*", 
	[
	(PF_FONT, "font", "Font:", "RothenBurg Decorative"),
	(PF_STRING, "yourname", "Your Name:", "Gimp Learn"),
	(PF_GRADIENT, "gradient", "Gradient:", 0),
	(PF_TOGGLE, "reversegradient",   "Reverse Gradient?:", 0),
	],
	[],
	python_tt_pat_mandala)

main()
