#!/usr/bin/env python

# draw_average_color_from_source_layer.py Release 8
# This script is to aid in creating polygon art,
# It will get average color of selected area on layer called "Source" and fill selected area with that color on active layer.
# Created by Tin Tran http://gimplearn.net
# Comments directed to http://gimplearn.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Subtract [Alpha To Selection of current layer] before applying average color
# Rel 3: Work with multi-selections (each area with its own average color)
# Rel 4: Add option to grow selection of each pathed area by number of pixels to intersect with original selection
#        to get pixel accuracy.
# Rel 5: Got rid of undo-group call and use GIMP 2.10 drawable_edit_fill instead of old method (Also reducing Undo history cache to 16MB seemed to speed it up).
# Rel 6: Use "Paint-brush instead of Edit-fill" option for speed.
# Rel 7: Use "Pencil instead of Edit-fill" option for speed and dodge filled area option (NO if your selection does not have filled areas).
# Rel 8: Crossing my fingers. I think I got this time with speed enhancement by using Edit-Blend instead of other methods.
#import random
#import string
#import Image
from gimpfu import *
#from array import array
#import sys

def tt_average_fill_it(img,src_layer,work_layer,brushit):
	#Apply Average Color from Source if there is selection
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(img)
	if non_empty:
		#pdb.gimp_message("grabbing colors")
		r, _, _, _, _, _ = pdb.gimp_drawable_histogram(src_layer,HISTOGRAM_RED,0,1)
		g, _, _, _, _, _ = pdb.gimp_drawable_histogram(src_layer,HISTOGRAM_GREEN,0,1)
		b, _, _, _, _, _ = pdb.gimp_drawable_histogram(src_layer,HISTOGRAM_BLUE,0,1)
		pdb.gimp_context_set_foreground((int(r),int(g),int(b)))
		#pdb.gimp_message("grabbing colors done")
		pdb.gimp_context_set_background((int(r),int(g),int(b)))
		#pdb.gimp_edit_fill(work_layer,FOREGROUND_FILL)
		if brushit == 1:
			#pdb.gimp_edit_bucket_fill_full(work_layer,BUCKET_FILL_FG,LAYER_MODE_NORMAL,100,15.0,FALSE,FALSE,SELECT_CRITERION_COMPOSITE,x1,y1)
			#pdb.gimp_message("gimp_pencil")
			pdb.gimp_edit_blend(work_layer,BLEND_FG_BG_RGB,LAYER_MODE_NORMAL,GRADIENT_LINEAR,100,0,REPEAT_NONE,FALSE,FALSE,3,0.2,FALSE,x1,y1,x2,y2)
			#pdb.gimp_pencil(work_layer,2,[(x2+x1)/2,(y2+y1)/2])
			#pdb.gimp_message('gimp_pencil done')
		else:	
			pdb.gimp_drawable_edit_fill(work_layer,FILL_FOREGROUND)
def python_tt_draw_average(image, layer, grow, brushit, dodge):
	#pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	source_layer = pdb.gimp_image_get_layer_by_name(image,"Source")
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	if not non_empty:
		pdb.gimp_selection_all(image)
	
	#save selection to intersect with each grown area later
	saved_selection = pdb.gimp_selection_save(image)
	
	#deal with multi_selections, turns it into a path then get it's strokes
	pdb.plug_in_sel2path(image,layer)
	vectors = image.vectors[0]
	num_strokes,stroke_ids = pdb.gimp_vectors_get_strokes(vectors)
	
	if brushit == 1: #if brush it we select brush and set brushsize
		pdb.gimp_context_set_brush("2. Hardness 100")
		pdb.gimp_context_set_opacity(100)
		#pdb.gimp_context_set_brush_size(brushsize)
		pdb.gimp_context_get_gradient('FG to BG (Hardedge)')

	#loop through all strokes, select each stroke and do average color on each stroke
	for stroke_id in stroke_ids:
		temp_vectors = pdb.gimp_vectors_new(image,"Temp")
		pdb.gimp_image_insert_vectors(image,temp_vectors,None,0)
		
		_type, _num_points, _control_points, _closed = pdb.gimp_vectors_stroke_get_points(vectors,stroke_id)
		new_stroke_id = pdb.gimp_vectors_stroke_new_from_points(temp_vectors,_type,_num_points,_control_points,_closed)
		
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,temp_vectors)
		#grow it then insect with original selection to get exact select area to pixels accuracy
		pdb.gimp_selection_grow(image,grow)
		pdb.gimp_image_select_item(image,CHANNEL_OP_INTERSECT,saved_selection)
		#Try to Subtract [Alpha To Selection of current layer] before applying average color
		if dodge == 1:
			pdb.gimp_image_select_item(image,CHANNEL_OP_SUBTRACT,layer)
		tt_average_fill_it(image,source_layer,layer,brushit)
		#we're done, remove it
		pdb.gimp_image_remove_vectors(image,temp_vectors)

	pdb.gimp_image_remove_vectors(image,vectors)
	pdb.gimp_image_remove_channel(image,saved_selection)

	#UNCOMMENT BELOW WHEN DONE
	pdb.gimp_context_pop()
	#pdb.gimp_image_undo_group_end(image)
	
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_draw_average",                           
	"Fills selection of active layer with average color of selection on layer named 'Source'",
	"Fills selection of active layer with average color of selection on layer named 'Source'",
	"Tin Tran",
	"Tin Tran",
	"July 2017",
	"<Image>/Python-Fu/Fill Average Color..",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "grow", "Pixels to grow each area before intersecting with original selection:", 2, (0, 100, 1)),
	(PF_TOGGLE, "brushit", "Edit-Blend instead of Edit-Fill (Edit-blend is faster):", 1),
	#(PF_SPINNER, "brushsize", "Brush Size:", 500, (100, 10000, 1)),
	(PF_TOGGLE, "dodge", "Dodge filled areas (Subtract alpha to selection's selection):", 0),
	],
	[],
	python_tt_draw_average)

main()