#!/usr/bin/env python

# filled_text_selection.py Release 11
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Fix error for when there's only one line of text with no linebreaks.
# Rel 3: Strip leading/trailing whitespaces for each line (just in case user has whitespaces).
# Rel 4: Added Scale last line Toggle for sometimes you don't want to scale last line when it has too few characters.
# Rel 5: Last line scale factor calculation fix.
# Rel 6: Even better last line scale factor.
# Rel 7: Irregular selection shape support.
# Rel 8: Use scale_path of font instead of scaling font, better quality (Suggested by ofnuts).
# Rel 9: Last Line scale fix.
# Rel 10: Processes empty lines indication paragraph (TO BE USED with version 2 of Prepare Text Into Selection Plug-in)
# Rel 11: Added "Did you use Prepare Text In Selection to Prepare this text TOGGLE (to use same height as prepared)
from gimpfu import *
import random
#from array import array
#import sys

def python_tt_filled_text_selection(image, layer, prepare, scale_last):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#core code goes here/below
	if not pdb.gimp_item_is_text_layer(layer):
		pdb.gimp_message("Layer must be a text layer!")
	else: #it's a text layer we're good to go
		non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
		if not non_empty:
			pdb.gimp_message("Must have an active selection")
		else:
			saved_selection = pdb.gimp_selection_save(image)
			sel_width = x2 - x1
			sel_height = y2 - y1
			text = pdb.gimp_text_layer_get_text(layer)
			texts = text.split("\n")
			font = pdb.gimp_text_layer_get_font(layer)
			fontsize,unit = pdb.gimp_text_layer_get_font_size(layer)
			#fontsize *= 2.0 #so when we scale down it still looks good
			width,height,ascent,descent = pdb.gimp_text_get_extents_fontname(texts[0],fontsize,unit,font)
			current_y = 0

			line = 0
			last_line = len(texts)
			max_width = 0
			min_width = 1000000
			#make a layer to render font's paths to
			result_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"Filled_Text_Selection_result",100,LAYER_MODE_NORMAL)
			pdb.gimp_image_insert_layer(image,result_layer,None,0)

			#loop through to find max width first and min_width (excluding last line)
			for text in texts:
				line += 1
				_width,_height,_ascent,_descent = pdb.gimp_text_get_extents_fontname(text,fontsize,unit,font)
				max_width = max(_width,max_width)
				if line < last_line:
					min_width = min(_width,min_width)

			line = 0
			max_scale = 0
			min_scale = 1000000
			for ti in range(0,len(texts)):
				text = texts[ti]
				is_last_line = False
				text = text.strip()
				line += 1
				if text.strip() != "": #if line is not blank process it, otherwise just move onto next one
					
					if (line == last_line):
						is_last_line = True
					else: #we're not last line we can check next to see if it's blank indication that 
					      #cur_line is last line on this paragraph
						if (texts[ti+1].strip() == ""):
							is_last_line = True
					# caculate selection for each line
					if prepare == 1:
						line_selection_height = _height
					else:	
						line_selection_height = sel_height*1.0/last_line
					pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,saved_selection)
					selection_height_at = (line - 1)*line_selection_height+(line_selection_height/2.0)
					pdb.gimp_image_select_rectangle(image,CHANNEL_OP_INTERSECT,0,y1+selection_height_at,image.width,2)
					_non_empty,_x1,_y1,_x2,_y2 = pdb.gimp_selection_bounds(image)
					_sel_width = _x2-_x1
					_sel_height = _y2-_y1

					fontname = pdb.gimp_text_fontname(image,None,0,0,text,0,TRUE,fontsize,unit,font)
					font_vectors = pdb.gimp_vectors_new_from_text_layer(image,fontname)
					pdb.gimp_image_insert_vectors(image,font_vectors,None,0)
					_width,_height,_ascent,_descent = pdb.gimp_text_get_extents_fontname(text,fontsize,unit,font)

					scale_factor_x = _sel_width*1.0/max_width
					scale_x = scale_factor_x * ((max_width*1.0)/_width)
					scale_factor_y = line_selection_height*1.0/_height
					if not is_last_line: #ignore max_min scales when we do last_line or last line on paragraph
						max_scale = max(max_scale,scale_x)
						min_scale = min(min_scale,scale_x)
					if not is_last_line or (scale_last == 1):
						#record max_scale and min_scale to use for last line when user says No to scaling		
						scaled_vectors = pdb.gimp_item_transform_scale(font_vectors,0,0,image.width*scale_x,image.height*scale_factor_y)
						#pdb.gimp_item_transform_scale(merged_result,_x1,current_y,_x1+_sel_width,current_y+line_selection_height)
					else: #last line and scale_last == 0
						#scale it based on previous lines.
						#use average scale (average of min and max) or scale to fit whichever is smaller.
						scale_factor_x = min((max_scale+min_scale)/2.0,scale_x)
						scaled_vectors = pdb.gimp_item_transform_scale(font_vectors,0,0,image.width*scale_factor_x,image.height*scale_factor_y)
						#pdb.gimp_item_transform_scale(merged_result,_x1,current_y,_x1+(scale_factor*_sel_width),current_y+line_selection_height)			
					#if line > 1: #starting at text layer 2, keep merging down.
						#merged_result = pdb.gimp_image_merge_down(image,merged_result,EXPAND_AS_NECESSARY)
					#render vectors
					translate_vectors = pdb.gimp_item_transform_translate(scaled_vectors,_x1,y1+current_y)
					pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,translate_vectors)
					pdb.gimp_edit_fill(result_layer,FILL_FOREGROUND)
					pdb.gimp_image_remove_vectors(image,translate_vectors)
					pdb.gimp_image_remove_layer(image,fontname)
				current_y += line_selection_height
			pdb.gimp_selection_none(image)
			#pdb.gimp_item_transform_scale(merged_result,x1,y1,x2,y2)
			#pdb.gimp_item_set_name(merged_result,"Filled Text Selection")

	#UNCOMMENT BELOW WHEN DONE
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_filled_text_selection",                           
	"Fill selection with chunk of text",
	"Fill selection with chunk of text",
	"Tin Tran",
	"Tin Tran",
	"March 2019",
	"<Image>/Python-Fu/Filled Text Selection...",             #Menu path
	"RGB*, GRAY*", 
	[
	#(PF_SPINNER, "frame_percent", "Frame percentage (percent to grow by to create border):", 10, (5, 100, 1)),
	#(PF_TOGGLE, "keep_aspect", "Keep Aspect Ratio of Border", 1),
	(PF_TOGGLE, "prepare","Did you use Prepare Text Into Selection to Prepare this text:", 1),
	(PF_TOGGLE, "scale_last","Scale Last Line (Sometimes it's too short, you don't want to scale it):", 1),
	],
	[],
	python_tt_filled_text_selection)

main()