#!/usr/bin/env python

# Get Curves Rel 1
# Created by Tin Tran 
# Comments directed to http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
from gimpfu import *
import os
from array import array
def outchannel(file,channel,channel_values):
	file.write("(channel " + channel + ")\n")
	file.write("(curve\n")
	file.write("    (curve-type free)\n")
	file.write("    (n-points 17)\n")
	file.write("    (points 34")
	for i in range(0,34):
		file.write(" -1.000000")
	file.write(")\n")
	file.write("    (n-samples 256)\n")
	file.write("    (samples 256")
	for i in range(0,256):
		file.write(" "+str(float(channel_values[i])))
	file.write("))\n")
	
def outfile(path, filename, r, g, b):
	file = open(path + os.sep + filename, "w")
	file.write("# GIMP curves tool settings\n")
	file.write("\n")
	file.write("(time 0)\n")
	
	#We don't do anythhing with value and alpha we only want RGB
	#So set value and alpha as default straight curve
	value = []
	alpha = []
	for i in range(0,256):
		value.append(float(i)/255)
		alpha.append(float(i)/255)
		
	outchannel(file,"value",value)
	outchannel(file,"red",r)
	outchannel(file,"green",g)
	outchannel(file,"blue",b)
	outchannel(file,"alpha",alpha)
	
	file.write("\n# end of curves tool settings\n")
	file.close()
def get_pixel(x,y,image_width,p_size,src_pixels) :
	src_pos = (x + image_width * y) * p_size
	return (src_pixels[src_pos: src_pos + p_size])
	
def python_get_curves_tt(image, layer, path, filename):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	r,g,b = [0]*257,[0]*257,[0]*257 #we don't care if the extra value [256] is in there, no error if int color is higher than 255
	rf,gf,bf = [0]*257,[0]*257,[0]*257 #frequencies
	first_layer = image.layers[0]
	second_layer = image.layers[1]
	firstRgn = first_layer.get_pixel_rgn(0, 0, first_layer.width, first_layer.height,False, False)
	first_pixels = array("B", firstRgn[0:first_layer.width, 0:first_layer.height])
	secondRgn = second_layer.get_pixel_rgn(0, 0, second_layer.width, second_layer.height,False, False)
	second_pixels = array("B", secondRgn[0:second_layer.width, 0:second_layer.height])
	p_size = len(firstRgn[0,0])
	for y in range(0,first_layer.height):
		for x in range(0,first_layer.width):
			#use pixel range is faster, hopefully
			pixel1 = get_pixel(x,y,first_layer.width,p_size,first_pixels)
			pixel2 = get_pixel(x,y,second_layer.width,p_size,second_pixels)
			#num_channels1,pixel1 = pdb.gimp_drawable_get_pixel(first_layer,x,y)
			#num_channels2,pixel2 = pdb.gimp_drawable_get_pixel(second_layer,x,y)
			c = pixel2[0]
			r[c] += pixel1[0]
			rf[c] +=1
			c = pixel2[1]
			g[c] += pixel1[1]
			gf[c] +=1
			c = pixel2[2]
			b[c] += pixel1[2]
			bf[c] +=1
			pdb.gimp_progress_update(float(y)/first_layer.height)
	for i in range(0,256):
		if (rf[i] > 0): 
			r[i] = r[i]*1.0/(rf[i]*255.0)
		if (gf[i] > 0): 
			g[i] = g[i]*1.0/(gf[i]*255.0)
		if (bf[i] > 0): 
			b[i] = b[i]*1.0/(bf[i]*255.0)
	outfile(path,filename,r,g,b)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_get_curves_tt",                           
	"Get RGB Curves",
	"Get RGB Curves",
	"Tin Tran",
	"Tin Tran",
	"March 2017",
	"<Image>/Python-Fu/Get RGB Curves...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_DIRNAME, "path", "Save Directory/Path:", 0),
	(PF_STRING, "filename", "Save Filename:", "RGB_curve.gcf"),
	],
	[],
	python_get_curves_tt)

main()
