#!/usr/bin/env python

# Get Curves Rel 5
# Created by Tin Tran 
# Comments directed to http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: uses histogram so it's a little quicker
# Rel 3: super fast by sampling only 16 points
# Rel 4: allow channel selection
# rel 5: allow curve type selection
from gimpfu import *
import os
from array import array
def outchannel(file,channel,channel_values,type):
	file.write("(channel " + channel + ")\n")
	file.write("(curve\n")
	file.write("    (curve-type " + type + ")\n")
	file.write("    (n-points 17)\n")
	file.write("    (points 34")
	if type == "free":
		for i in range(0,34):
			file.write(" -1.000000")
	else: #try to use smooth by writing out points on curve
		chunks = 16
		x_diff = 1.0/chunks
		for i in range(0,17):
			x = i * x_diff
			file.write(" " + str(x))
			y = channel_values[int(i*(255.0/chunks))]
			file.write(" " + str(y))
	file.write(")\n")
	file.write("    (n-samples 256)\n")
	file.write("    (samples 256")
	for i in range(0,256):
		file.write(" "+str(float(channel_values[i])))
	file.write("))\n")
	
def outfile(path, filename, v, r, g, b, type):
	file = open(path + os.sep + filename, "w")
	file.write("# GIMP curves tool settings\n")
	file.write("\n")
	file.write("(time 0)\n")
	if type=="smooth":
		file.write("(linear no)\n");
	#We don't do anythhing with value and alpha we only want RGB
	#So set value and alpha as default straight curve
	alpha = []
	for i in range(0,256):
		alpha.append(float(i)/255)
	if len(v) == 0: v = alpha
	if len(r) == 0: r = alpha
	if len(g) == 0: g = alpha
	if len(b) == 0: b = alpha
	outchannel(file,"value",v,type)
	outchannel(file,"red",r,type)
	outchannel(file,"green",g,type)
	outchannel(file,"blue",b,type)
	outchannel(file,"alpha",alpha,type)
	
	file.write("\n# end of curves tool settings\n")
	file.close()
def findlow(values,freqs,index):
	for j in range(index-1,-1,-1):
		if freqs[j] > 0:
			return (values[j],j)
	return (0.0,-1) #if we don't find anything we return 0 and index of -1		
def findhigh(values,freqs,index):
	for j in range(index+1,256):
		if freqs[j] > 0:
			return (values[j],j)
	return (1.0,256)#if we don't find anything we return 1.00 and index of 256
def python_get_curves_tt(image, layer, path, filename, channel, type):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	v,r,g,b = [0]*257,[0]*257,[0]*257,[0]*257 #we don't care if the extra value [256] is in there, no error if int color is higher than 255
	vf,rf,gf,bf = [0]*257,[0]*257,[0]*257,[0]*257 #frequencies -1 to mark that it hasn't been set.
	first_layer = image.layers[0]
	second_layer = image.layers[1]
	
	pdb.gimp_context_set_antialias(True)
	pdb.gimp_context_set_feather(False)
	pdb.gimp_context_set_feather_radius(0,0)
	pdb.gimp_context_set_sample_threshold(0.0)
	pdb.gimp_context_set_sample_transparent(False)
	if channel == 1: #RGB channels
		pdb.gimp_progress_init("Collecting Red Statistics...",None)
		pdb.gimp_context_set_sample_criterion(SELECT_CRITERION_R)
		for i in range(0,16):
			intensity = i * 16
			#Red Channel Stats
			pdb.gimp_image_select_color(image,CHANNEL_OP_REPLACE,second_layer,(intensity,0,0))
			mean2,std_dev,median,pixels2,count2,percentile = pdb.gimp_histogram(second_layer,HISTOGRAM_RED,0,255)
			mean,std_dev,median,pixels,count,percentile = pdb.gimp_histogram(first_layer,HISTOGRAM_RED,0,255)
			c = int(mean2)
			r[c] += mean
			rf[c] += 1
			pdb.gimp_progress_update(float(intensity)/255.0)
		pdb.gimp_progress_init("Collecting Green Statistics...",None)
		pdb.gimp_context_set_sample_criterion(SELECT_CRITERION_G)	
		for i in range(0,16):
			intensity = i * 16
			#Green Channel Stats
			pdb.gimp_image_select_color(image,CHANNEL_OP_REPLACE,second_layer,(0,intensity,0))
			mean2,std_dev,median,pixels2,count2,percentile = pdb.gimp_histogram(second_layer,HISTOGRAM_GREEN,0,255)
			mean,std_dev,median,pixels,count,percentile = pdb.gimp_histogram(first_layer,HISTOGRAM_GREEN,0,255)
			c = int(mean2)
			g[c] += mean
			gf[c] += 1
			pdb.gimp_progress_update(float(intensity)/255.0)
		pdb.gimp_progress_init("Collecting Blue Statistics...",None)	
		pdb.gimp_context_set_sample_criterion(SELECT_CRITERION_B)
		for i in range(0,16):
			intensity = i * 16
			#Blue Channel Stats
			pdb.gimp_image_select_color(image,CHANNEL_OP_REPLACE,second_layer,(0,0,intensity))
			mean2,std_dev,median,pixels2,count2,percentile = pdb.gimp_histogram(second_layer,HISTOGRAM_BLUE,0,255)
			mean,std_dev,median,pixels,count,percentile = pdb.gimp_histogram(first_layer,HISTOGRAM_BLUE,0,255)
			c = int(mean2)
			b[c] += mean
			bf[c] += 1
			pdb.gimp_progress_update(float(intensity)/255.0)
	else: #VALUe channel
		pdb.gimp_progress_init("Collecting Value Statistics...",None)	
		pdb.gimp_context_set_sample_criterion(SELECT_CRITERION_V)
		for i in range(0,16):
			intensity = i * 16
			#Blue Channel Stats
			pdb.gimp_image_select_color(image,CHANNEL_OP_REPLACE,second_layer,(intensity,intensity,intensity))
			mean2,std_dev,median,pixels2,count2,percentile = pdb.gimp_histogram(second_layer,HISTOGRAM_VALUE,0,255)
			mean,std_dev,median,pixels,count,percentile = pdb.gimp_histogram(first_layer,HISTOGRAM_VALUE,0,255)
			c = int(mean2)
			v[c] += mean
			vf[c] += 1
			pdb.gimp_progress_update(float(intensity)/255.0)
	pdb.gimp_selection_none(image)	
		
	for i in range(0,256):
		if (vf[i] > 0): 
			v[i] = v[i]*1.0/(vf[i]*255.0)
		if (rf[i] > 0): 
			r[i] = r[i]*1.0/(rf[i]*255.0)
		if (gf[i] > 0): 
			g[i] = g[i]*1.0/(gf[i]*255.0)
		if (bf[i] > 0): 
			b[i] = b[i]*1.0/(bf[i]*255.0)
	
	#GIMp 2.10 hack for when some values aren't set because color select is weird
	#We set value based on previous value (low) and next value (high) found in our arrays
	for i in range(0,256):
		if vf[i] == 0: #hasn't been set
			low,low_index = findlow(v,vf,i)
			high,high_index = findhigh(v,vf,i)
			v[i] = low + float(high-low)/(high_index-low_index) * (i-low_index)
		if rf[i] == 0: #hasn't been set
			low,low_index = findlow(r,rf,i)
			high,high_index = findhigh(r,rf,i)
			r[i] = low + float(high-low)/(high_index-low_index) * (i-low_index)
		if gf[i] == 0: #hasn't been set
			low,low_index = findlow(g,gf,i)
			high,high_index = findhigh(g,gf,i)
			g[i] = low + float(high-low)/(high_index-low_index) * (i-low_index)	
		if bf[i] == 0: #hasn't been set
			low,low_index = findlow(b,bf,i)
			high,high_index = findhigh(b,bf,i)
			b[i] = low + float(high-low)/(high_index-low_index) * (i-low_index)
	if type == 0:
		curve_type = "smooth"
	else:
		curve_type = "free"
	if channel == 0:
		outfile(path,filename,v,[],[],[],curve_type)
	else:
		outfile(path,filename,[],r,g,b,curve_type)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_get_curves_tt",                           
	"Get RGB Curves",
	"Get RGB Curves",
	"Tin Tran",
	"Tin Tran",
	"June 2018",
	"<Image>/Python-Fu/Get RGB Curves...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_DIRNAME, "path", "Save Directory/Path:", 0),
	(PF_STRING, "filename", "Save Filename:", "RGB_curve.gcf"),
	(PF_OPTION,"channel",   "Channel(s):", 1, ["Value","RGB"]),
	(PF_OPTION,"type",   "Curve Type:", 0, ["Smooth","Freehand"]),
	],
	[],
	python_get_curves_tt)

main()
