#!/usr/bin/env python

# growing-brush-on-path.py
# Created by Tin Tran http://gimplearn.nte
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# --------------------------------------------
# Paints brush following a path with growing size.


import math
from gimpfu import *
#from array import array
import os, string, sys
import random

#Copied this from ofn-text-along-path.py
def computeOrientedSlope(dx,dy,slope): #This is a better version than below computeThetaWithSlope function.
	if abs(slope) > 100000: # very vertical
		return math.atan2(dy,dx) # no perfect, but properly oriented
        # keep dx/dy signs but give then the same ratio as in slope
	return math.atan2(math.copysign(slope,dy),math.copysign(1,dx))

def python_tt_growing_brush_on_path(image, layer, startpercentage, endpercentage, brushes) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#PUT YOUR CODE HERE 
	context_brush_size = pdb.gimp_context_get_brush_size()
	context_brush_angle = pdb.gimp_context_get_brush_angle()
	total_dist = 0.0
	brushes = int(brushes)
	

	active_vectors = pdb.gimp_image_get_active_vectors(image)
	num_strokes, stroke_ids = pdb.gimp_vectors_get_strokes(active_vectors)
	strokeid = stroke_ids[0]
	vectorlength = pdb.gimp_vectors_stroke_get_length(active_vectors,strokeid,0.01)
	brushes = int(vectorlength/100.0 * float(brushes))
	fract = float(endpercentage - startpercentage) / (brushes-1)
	locates = [0.2];
	for i in range(0,brushes): #get total dist
		total_dist += startpercentage
		total_dist += i*fract
	cur_dist = 0.0	
	for i in range(0,brushes): #dist at
		cur_dist += startpercentage
		cur_dist += i*fract
		dist_at = cur_dist/total_dist
		locates.append(dist_at*vectorlength)

	for i in range(0,len(locates)):
		pdb.gimp_progress_update(float(i)/len(locates))
		dist = locates[i] - 0.1
		x_point,y_point,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,strokeid,dist,0.01)
		x_point1,y_point1,slope1,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,strokeid,dist-0.05,0.01)
		x_point2,y_point2,slope2,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,strokeid,dist+0.05,0.01)
			
		# x_point1,y_point1,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,strokeid,dist1,0.001)
		# x_point2,y_point2,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,strokeid,dist2,0.001)
		angle = computeOrientedSlope(x_point2 - x_point1, y_point2 - y_point1, slope)
		brush_size = (startpercentage+(fract * i))/100.0 
		brush_size = brush_size * context_brush_size;
		if brush_size < 1.0:
			brush_size = 1.0
		pdb.gimp_context_set_brush_size(brush_size) #set brush size to reflect size change
		angle = math.degrees(angle)
		setangle = context_brush_angle + angle
		if setangle > 180:
			setangle = setangle - 360
		if setangle < -180:
			setangle = setangle + 360	
		pdb.gimp_context_set_brush_angle(setangle)
		pdb.gimp_paintbrush(layer,0,2,(x_point2,y_point2),PAINT_CONSTANT,0)	
	
	
	# pdb.gimp_context_set_brush_size(context_brush_size * size_ratio) #set brush size to reflect size change
	# pdb.gimp_paintbrush(layer,0,2,(x,y),PAINT_CONSTANT,0)
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return


register(
	"python_tt_growing_brush_on_path",
	"Growing Brush on Path with Active Brush",
	"Growing Brush on Path with Active Brush...",
	"Tin Tran",
	"Tin Tran",
	"December 2020",
	"<Image>/Python-Fu/Growing Brush on Path...",
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "startpercentage", "Start Percentage (Brush size at starting):", 10, (1, 500, 1)),
	(PF_SPINNER, "endpercentage", "End Percentage (Brush size at end):", 100, (0, 500, 1)),
	(PF_SPINNER, "brushes", "Brushes Per 100 pixels (average):", 2.0, (0.1, 500, 0.1)),
	],
	[],
	python_tt_growing_brush_on_path)

main()