#!/usr/bin/env python

# jiggle-visible-layers.py
# Request by SweenyMark:https://gimplearn.net/viewtopic.php?f=18&t=2604
# Created by Tin Tran https://gimplearn.net
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Minimum to 1
# Rel 3: Look group layergroups to find all visible layers
import math
import string
#import Image
from gimpfu import *
from array import array
import sys
import random
def findchildren(layer,sources): #append to sources if layers start with label
	if (len(layer.children) > 0): #if it has children look through it's child
		for l in layer.children:
			if (l.visible and len(l.children)==0):
				sources.append(l)
			findchildren(l,sources) #recursive check to see if that's a layer group	

def python_jiggle_visible_layers(image, layer,xscale,xscalejig,yscale,yscalejig,rot,rotjig,x,xjig,y,yjig):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	sources = []
	for layer in image.layers:
		if (layer.visible and len(layer.children)==0): #if it's visible and it has no children then it's a content layer
			sources.append(layer)
		findchildren(layer,sources)
	for layer in sources: #Loop through sources layers which are all visble layers we found
		#if layer.visible:      #only apply to visible layers
			#apply scale x and scale y with jiggles
		new_width = (xscale + random.randrange(xscalejig*2) - xscalejig)/100.0 * layer.width
		new_height = (yscale + random.randrange(yscalejig*2) - yscalejig)/100.0 * layer.height
		pdb.gimp_layer_scale_full(layer,new_width,new_height,TRUE,INTERPOLATION_CUBIC)
		rotation = (rot + random.randrange(rotjig*2) - rotjig)
		rad_rotation = rotation/360.0 * 2 * math.pi
		pdb.gimp_item_transform_rotate(layer,rad_rotation,TRUE,0,0)
		shift_x = x + random.randrange(xjig*2) - xjig
		shift_y = y + random.randrange(yjig*2) - yjig
		pdb.gimp_item_transform_translate(layer,shift_x,shift_y)

	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_fu_jiggle_visible_layers_tt",
	"Jiggle Visible Layers",
	"Jiggle Visible Layers",
	"Tin Tran",
	"Tin Tran",
	"December 2019",
	"<Image>/Python-Fu/Jiggle Visible Layers",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "xscale", "Scale X Percentage:", 100, (50, 200, 1)),
	(PF_SPINNER, "xscalejig", "Scale X Jiggle Percentage", 5, (1,100, 1)),
	(PF_SPINNER, "yscale", "Scale Y Percentage:", 100, (50, 200, 1)),
	(PF_SPINNER, "yscalejig", "Scale Y Jiggle Percentage", 5, (1,100, 1)),
	(PF_SPINNER, "rot", "Rotation angle:", -20, (-180, 180, 1)),
	(PF_SPINNER, "rotjig", "Rotation jiggle angle:", 5, (1,360, 1)),
	(PF_SPINNER, "x", "X Shift Pixels:", 0, (-1000, 1000, 1)),
	(PF_SPINNER, "xjig", "X Shift Jiggle Pixels", 5, (1,500, 1)),
	(PF_SPINNER, "y", "Y Shift Pixels:", 0, (-1000, 1000, 1)),
	(PF_SPINNER, "yjig", "Y Shift Jiggle Pixels", 5, (1,500, 1)),
	#(PF_COLOR, "toalpha", "Color to Make alpha (to turn image into silhoutte):", (255, 255, 255)),
	#(PF_COLOR, "light", "Icon Color:", (255, 156, 71)),
	#(PF_COLOR, "dark", "Icon Slightly Darker Color:", (183, 110, 51)),
	],
	[],
	python_jiggle_visible_layers)


main()
