#!/usr/bin/env python

# Label_Number_From_Path_Points Rel 1
# Created by Tin Tran
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.

import math
import string
#import Image
from gimpfu import *
from array import array
def draw_numbers(image,fill_layer,number_layer,x1,y1,x2,y2,x,y,radius,fill_circle,write_number,number,font):
	#x1,y1,x2,y2 are selection bounds
	#x,y is the point where we want to find surrounding rectangle
	ret_value = 0
	if (x >= x1) and (x <= x2) and (y >= y1) and (y <= y2): #only do this if point is within selection bounds
		if fill_circle == 1 or write_number == 1:
			pdb.gimp_selection_none(image);
			
		pdb.gimp_image_select_ellipse(image,CHANNEL_OP_ADD,x-radius,y-radius,radius*2,radius*2)
		if fill_circle == 1:
			pdb.gimp_edit_fill(fill_layer,BACKGROUND_FILL)
	    
		if write_number == 1:
			pdb.gimp_selection_none(image)
			text_layer = pdb.gimp_text_layer_new(image,str(number),font,200,0)
			pdb.gimp_image_insert_layer(image,text_layer,None,-1)
			
			#when text is created it's not exactly centered in layer sometimes spaces around it is uneven
			#So here we'll crop layer so that we're left with only the text, this way it'll be dead center in our circle
			#pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,text_layer)
			pdb.plug_in_autocrop_layer(image,text_layer)
			
			#calculate ratio to transform text into circle
			text_width = pdb.gimp_drawable_width(text_layer)
			text_height = pdb.gimp_drawable_height(text_layer)
			width_over_height_ratio = (text_width * 1.0)/text_height
			
			if width_over_height_ratio <= 1:
				halfheight = radius/2
				halfwidth = halfheight * width_over_height_ratio
			else:
				halfwidth = radius/2
				halfheight = halfwidth / width_over_height_ratio
				
			pdb.gimp_item_transform_perspective(text_layer,x-halfwidth,y-halfheight,x+halfwidth,y-halfheight,x-halfwidth,y+halfheight,x+halfwidth,y+halfheight)
			pdb.gimp_image_merge_down(image,text_layer, EXPAND_AS_NECESSARY)
			ret_value = 1 #we drew a number we return 1 to use as increment
	return ret_value
			
def python_label_numbers_from_path_points(image, layer, radius, fill_circle, write_number,font ) :
	pdb.gimp_image_undo_group_start(image)
	
	#grab selection bounding box values
	selection = pdb.gimp_selection_bounds(image);
	
	x1 = selection[1];
	y1 = selection[2];
	x2 = selection[3];
	y2 = selection[4];
	
	pdb.gimp_selection_none(image)
	
	#creates new layer for fill to be drawn on
	fill_layer = 0;
	fill_layer_name = "";
	if fill_circle == 1:
		fill_layer_name = "Draw Fill";
		width = pdb.gimp_image_width(image)
		height = pdb.gimp_image_height(image)
		fill_layer = pdb.gimp_layer_new(image,width,height,RGBA_IMAGE,fill_layer_name,100,NORMAL_MODE)
		pdb.gimp_image_insert_layer(image,fill_layer,None,-1)
		fill_layer_name = pdb.gimp_item_get_name(fill_layer)
	#creates new layer for number to be drawn on
	number_layer = 0;
	number_layer_name = "";
	if write_number == 1:
		number_layer_name = "Draw Numbers";
		width = pdb.gimp_image_width(image)
		height = pdb.gimp_image_height(image)
		number_layer = pdb.gimp_layer_new(image,width,height,RGBA_IMAGE,number_layer_name,100,NORMAL_MODE)
		pdb.gimp_image_insert_layer(image,number_layer,None,-1)
		number_layer_name = pdb.gimp_item_get_name(number_layer)
	
	
	#get active vectors and do some work
	active_vectors = pdb.gimp_image_get_active_vectors(image)
	num_strokes,stroke_ids = pdb.gimp_vectors_get_strokes(active_vectors)
	
	number = 1
	for stroke_id in stroke_ids:
		type,num_points,controlpoints,closed = pdb.gimp_vectors_stroke_get_points(active_vectors,stroke_id)
		#pdb.gimp_message(str(num_points))
		#pdb.gimp_message(str(controlpoints))
		
		for i in range(0,num_points/6): #each point has 3 control points x 2 coordinates (so 6 values)
			x = controlpoints[(i*6) + 2]  #third index is middle control point's x
			y = controlpoints[(i*6) + 3]  #fourth index is middle control point's y
			inc = draw_numbers(image,fill_layer,number_layer,x1,y1,x2,y2,x,y,radius,fill_circle,write_number,number,font)
			if write_number == 1: #if we write number we merged down so we gotta grab that layer pointer again
				number_layer = pdb.gimp_image_get_layer_by_name(image,number_layer_name) #after it's merged we gotta grab it again
			number = number+inc
	
	#if we draw anything we just select none else leave it as is as user just wants to make selection
	if fill_circle == 1 or write_number == 1:
		pdb.gimp_selection_none(image)
	
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_label_numbers_from_path_points",
	"Label Numbers Or Select Active Path Points specifying radius",
	"Label Numbers Or Select Active Path Points specifying radius...",
	"Tin Tran",
	"Tin Tran",
	"2016",
	"<Image>/Layer/Label Numbers From Active Path Points",
	"*",      # Create a new image, don't work on an existing one
	[
	(PF_SPINNER, "radius", "Label Radius:", 25, (1, 2000, 10)),
	(PF_TOGGLE, "fill_circle",   "Fill Circle With Background Color:", 1),
	(PF_TOGGLE, "write_number", "Write Numbers With Foreground Color:", 1),
	(PF_FONT, "number_font", "Number's Font:", 0)
	#(PF_COLOR, "black",  "Black point color",  (0,0,0) ),
	#(PF_COLOR, "white",  "White point color",  (255,255,255) ),
	#(PF_COLOR, "gray",  "Gray point color",  (128,128,128) )
	#(PF_FILE, "infilename", "Temp Filepath", "/Default/Path")
	#(PF_DIRNAME, "source_directory", "Source Directory", "") for some reason, on my computer when i(Tin) use PF_DIRNAME the pythonw.exe would crash
	],
	[],
	python_label_numbers_from_path_points)

main()
