#!/usr/bin/env python

# linocut_look.py Rel 2
# Created by Tin Tran
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
# Rel 2: Added different base, tip percentage options.
from gimpfu import *
import random
def python_tt_linocut_look(image, layer, samples, cuts, base_percentage, tip_percentage) : #FUNCTION DEFINITION
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	pdb.gimp_selection_none(image)
	#creates a working layer
	work_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,'work layer',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,work_layer,None,0)

	samples = int(samples)
	cuts = int(cuts)
	#get active vector and work with it
	vectors_list = []
	#only do it for visible vectors
	for vector in list(image.vectors)[::-1]:
		if pdb.gimp_item_get_visible(vector):
			vectors_list.append(vector)

	for vectors in vectors_list:
		#grab strokes
		num_strokes, stroke_ids = pdb.gimp_vectors_get_strokes(vectors)
		stroke1 = stroke_ids[0]
		stroke2 = stroke_ids[1]
		length1 = pdb.gimp_vectors_stroke_get_length(vectors,stroke1,0.01)
		length2 = pdb.gimp_vectors_stroke_get_length(vectors,stroke2,0.01)
		section1 = float(length1-0.02)/samples
		section2 = float(length2-0.02)/samples
		sample1 = []
		sample2 = []
		#get sample points of the two vectors
		for sample in range(0,samples+1):
			dist1 = 0.01 + (sample * section1)
			dist2 = 0.01 + (sample * section2)
			x1,y1,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke1,dist1,0.01)
			x2,y2,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke2,dist2,0.01)
			sample1.append([x1,y1])
			sample2.append([x2,y2])
		#make a flat list of sample2 points but backward so we can add it polygon select later
		backwardpoints = sample2[::-1]
		bwp = [] #backwardpoint flatlist
		for point in backwardpoints:
			bwp.append(point[0])
			bwp.append(point[1])
		forwardpoints = sample1
		'''fwp = [] #backwardpoint flatlist
		for point in forwardpoints:
			fwp.append(point[0])
			fwp.append(point[1])
		pdb.gimp_image_select_polygon(image,CHANNEL_OP_REPLACE,len(fwp)+len(bwp),fwp+bwp)
			#fill color based on alternating cut
		pdb.gimp_edit_fill(work_layer,FILL_FOREGROUND)
		'''		
		#loop through for each cut, calculate where the points of each cut should be	
		for cut in range(0,cuts):
			fwp = [] #start cut as empty flatlist
			for sample in range(0,len(sample1)):
				cur_percentage = base_percentage/100.0 + float(tip_percentage - base_percentage)/100.0/(samples)*sample
				x1,y1 = sample1[sample][0],sample1[sample][1]
				x2,y2 = sample2[sample][0],sample2[sample][1]
				xdif = x2 - x1
				ydif = y2 - y1
				xsect = float(xdif)/cuts
				ysect = float(ydif)/cuts
				xcut = x1
				ycut = y1
				if cut % 2 == 0: #alternating grow and shrink based on percentage
					xcut += (cut * xsect) - (xsect*0.5) + (xsect * cur_percentage)
					ycut += (cut * ysect) - (ysect*0.5) + (ysect * cur_percentage)
				else:
					xcut += (cut * xsect) + (xsect*0.5) - (xsect * cur_percentage)
					ycut += (cut * ysect) + (ysect*0.5) - (ysect * cur_percentage)
				#append new cut point to forwardpoint flatlist
				fwp.append(xcut)
				fwp.append(ycut)
			pdb.gimp_image_select_polygon(image,CHANNEL_OP_REPLACE,len(fwp)+len(bwp),fwp+bwp)
			#fill color based on alternating cut
			if cut % 2 == 1:
				pdb.gimp_edit_fill(work_layer,FILL_FOREGROUND)
			else:
				pdb.gimp_edit_fill(work_layer,FILL_BACKGROUND)

	pdb.gimp_selection_none(image)
	#YOUR CODE ENDS ========================
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	#return

register(
	"python_fu_tt_linocut_look",
	"Make Linocut Look",
	"Make Linocut Look",
	"Tin Tran",
	"Tin Tran",
	"June 2019",
	"<Image>/Python-Fu/Linocut Look...",
	"*",      # Create a new image, don't work on an existing one
	[ 
	#INPUT
	(PF_SPINNER, "samples", "Sample Points (The more the smoother but slower):", 100, (10,5000,1)),
	(PF_SPINNER, "cuts", "Lino Cuts:", 10, (1,5000,1)),
	(PF_SPINNER, "base_percentage", "Base Percentage (Percentage of foreground color at start of strokes):", 50, (0,100,1)),
	(PF_SPINNER, "base_percentage", "Tip Percentage (Percentage of foreground color at end of strokes):", 50, (0,100,1)),
	#(PF_TOGGLE, "dropshadow", "Drop Shadow", 0),
	
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 10, (-4096,4096,0.1)),
	#(PF_SPINNER, "shadow_blur_radius", "Shadow Blur Radius:", 5, (0,1024,0.1)),
	#(PF_COLOR, "shadow_color", "Shadow Color:", (0,0,0)),
	#(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 50, (0,100,0.1)),
	#INPUT ENDS
	],
	[],
	python_tt_linocut_look)

main()