#!/usr/bin/env python

# lum_mask_dm.py
# following the steps described on this link's pdf by david: https://gimplearn.net/viewtopic.php?p=29419#p29419
# Created by Tin Tran https://gimplearn.net
# Comments directed to https://gimplearn.net
#
# After being created, that ungrateful wretch, david, modified it by adding layers LLLL.
# DDDD and replacing M with a medium dark (MD) and medium light (ML).
# Modified December 2019 to create full range of masks for low-contrast images.
# August 2020 Create channels of equal intensity range by adding a curve to channel
# used as source for masks.
# 2 September 2020 For/next loops for channel subtraction. Delete channels after last used.
# Increase to 10 layers with masks.
# 3 September 2020 Option added for 4, 6, 8, or 10 masked layers. Tidy-up of layer names.
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: DM 23 November 2019.
# Rel 3: DM 7 December 2019
# Rel 4: DM 31 August 2020
# Rel 5: DM 02 September 2020
# Rel 6: DM 03 September 2020
#import Image
from gimpfu import *
def python_lum_mask_dm(image, layer, nLayer=2) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)							#make drawable in order to desaturate (source, destination)
	pdb.gimp_image_insert_layer(image,layer_copy,None,0)								#(image, layer, parent, position)
	pdb.gimp_drawable_equalize(layer_copy, TRUE)										#equalize layer for maximum contrast
	pdb.gimp_drawable_desaturate(layer_copy,DESATURATE_LUMINANCE)						#(drawable, destination)

	#copy Red channel as La
	channel_La = pdb.gimp_channel_new_from_component(image,CHANNEL_RED,"L1")			#(image to add channel, component, channel name)
	pdb.gimp_image_insert_channel(image,channel_La,None,0)								#(image, channel, parent, position)
	
	splinePoints=[0.0,0.0,0.0625,0.125,0.125,0.250,0.250,0.375,0.5,0.5,0.75,0.625,0.875,0.750,0.9375,0.875,1.0,1.0]	#curve to equalise brightness range
	pdb.gimp_drawable_curves_spline(channel_La,HISTOGRAM_VALUE,len(splinePoints),splinePoints)				#between masks

	#create Da from Alpha subtract La							ALPHA = desaturated image(?)
	channel_Da = pdb.gimp_channel_new_from_component(image,CHANNEL_ALPHA,"Da")
	pdb.gimp_image_insert_channel(image,channel_Da,None,0)
	pdb.gimp_channel_combine_masks(channel_Da,channel_La,CHANNEL_OP_SUBTRACT,0,0)		#(channel1, channel2,operation, offsets x&Y)

	#create D2a from Alpha - La - La
	channel_D2a = pdb.gimp_channel_new_from_component(image,CHANNEL_ALPHA,"D2a")
	pdb.gimp_image_insert_channel(image,channel_D2a,None,0)
	for x in range(0,3):
		pdb.gimp_channel_combine_masks(channel_D2a,channel_La,CHANNEL_OP_SUBTRACT,0,0)

	#create D3a from Alpha - La - La - La												#required for layers 2 & 4 (option 0)
	channel_D3a = pdb.gimp_channel_new_from_component(image,CHANNEL_ALPHA,"D3a")
	pdb.gimp_image_insert_channel(image,channel_D3a,None,0)
	for x in range(0,4):
		pdb.gimp_channel_combine_masks(channel_D3a,channel_La,CHANNEL_OP_SUBTRACT,0,0)
	
	#create D4a from Alpha - La - La - La - La											#required for layers 4 & 6 (option 1)
	if nLayer >=0: 
		channel_D4a = pdb.gimp_channel_new_from_component(image,CHANNEL_ALPHA,"D4a")
		pdb.gimp_image_insert_channel(image,channel_D4a,None,0)
		for x in range(0,5):
			pdb.gimp_channel_combine_masks(channel_D4a,channel_La,CHANNEL_OP_SUBTRACT,0,0)
	
	#create D5a from Alpha - La - La - La - La - La										#required for layers 6 & 8 (option 2)
	if nLayer >=1:
		channel_D5a = pdb.gimp_channel_new_from_component(image,CHANNEL_ALPHA,"D5a")
		pdb.gimp_image_insert_channel(image,channel_D5a,None,0)
		for x in range(0,6):
			pdb.gimp_channel_combine_masks(channel_D5a,channel_La,CHANNEL_OP_SUBTRACT,0,0)

	#create D6a from Alpha - La - La - La - La - La - La								#required for layer 8 (option 2)
	if nLayer >=2:
		channel_D6a = pdb.gimp_channel_new_from_component(image,CHANNEL_ALPHA,"D6a")
		pdb.gimp_image_insert_channel(image,channel_D6a,None,0)
		for x in range(0,7):
			pdb.gimp_channel_combine_masks(channel_D6a,channel_La,CHANNEL_OP_SUBTRACT,0,0)
	
	#create D5 from Alpha - La - La - La - La - La - La									#required for layer 10 (option 3)
	if nLayer ==3:
		channel_D5 = pdb.gimp_channel_new_from_component(image,CHANNEL_ALPHA,"D5")
		pdb.gimp_image_insert_channel(image,channel_D5,None,0)
		for x in range(0,7):
			pdb.gimp_channel_combine_masks(channel_D5,channel_La,CHANNEL_OP_SUBTRACT,0,0)
			
	pdb.gimp_image_remove_channel(image,channel_La)
		
	#create L2a from RED_CHANNEL - Da
	channel_L2a = pdb.gimp_channel_new_from_component(image,CHANNEL_RED,"L2a")
	pdb.gimp_image_insert_channel(image,channel_L2a,None,0)
	pdb.gimp_channel_combine_masks(channel_L2a,channel_Da,CHANNEL_OP_SUBTRACT,0,0)

	#create L3a from RED_CHANNEL - Da - Da 												#required for layers 2 & 4 (option 0)
	channel_L3a = pdb.gimp_channel_new_from_component(image,CHANNEL_RED,"L3a")
	pdb.gimp_image_insert_channel(image,channel_L3a,None,0)
	for x in range(0,3):
		pdb.gimp_channel_combine_masks(channel_L3a,channel_Da,CHANNEL_OP_SUBTRACT,0,0)
	
	#create L4a from RED_CHANNEL - Da - Da - Da 										#required for layers 4 & 6 (option 1)
	if nLayer >=0:
		channel_L4a = pdb.gimp_channel_new_from_component(image,CHANNEL_RED,"L4a")
		pdb.gimp_image_insert_channel(image,channel_L4a,None,0)
		for x in range(0,4):
			pdb.gimp_channel_combine_masks(channel_L4a,channel_Da,CHANNEL_OP_SUBTRACT,0,0)
		
	#create L5a from RED_CHANNEL - Da - Da - Da - Da 									#required for layers 6 & 8 (option 2)
	if nLayer >=1:
		channel_L5a = pdb.gimp_channel_new_from_component(image,CHANNEL_RED,"L5a")
		pdb.gimp_image_insert_channel(image,channel_L5a,None,0)
		for x in range(0,5):
			pdb.gimp_channel_combine_masks(channel_L5a,channel_Da,CHANNEL_OP_SUBTRACT,0,0)

	#create L6a from RED_CHANNEL - Da - Da - Da - Da - Da								#required for layer 8 (option 2)
	if nLayer >=2:
		channel_L6a = pdb.gimp_channel_new_from_component(image,CHANNEL_RED,"L6a")
		pdb.gimp_image_insert_channel(image,channel_L6a,None,0)
		for x in range(0,6):
			pdb.gimp_channel_combine_masks(channel_L6a,channel_Da,CHANNEL_OP_SUBTRACT,0,0)
	
	#create L5 from RED_CHANNEL - Da - Da - Da - Da - Da - Da							#required for layer 10 (option 3)
	if nLayer ==3:
		channel_L5 = pdb.gimp_channel_new_from_component(image,CHANNEL_RED,"L5")
		pdb.gimp_image_insert_channel(image,channel_L5,None,0)
		for x in range(0,7):
			pdb.gimp_channel_combine_masks(channel_L5,channel_Da,CHANNEL_OP_SUBTRACT,0,0)
			
	pdb.gimp_image_remove_channel(image,channel_Da)

	#create D1 from D2a - D3a
	channel_D1 = pdb.gimp_channel_copy(channel_D2a)
	pdb.gimp_image_insert_channel(image,channel_D1,None,0)
	pdb.gimp_item_set_name(channel_D1,"D1")
	pdb.gimp_channel_combine_masks(channel_D1,channel_D3a,CHANNEL_OP_SUBTRACT,0,0)
	pdb.gimp_image_remove_channel(image,channel_D2a)

	#create D2 from D3a - D4a															#4 layers (option 0)
	channel_D2 = pdb.gimp_channel_copy(channel_D3a)
	pdb.gimp_image_insert_channel(image,channel_D2,None,0)
	pdb.gimp_item_set_name(channel_D2,"D2")
	pdb.gimp_channel_combine_masks(channel_D2,channel_D4a,CHANNEL_OP_SUBTRACT,0,0)
	pdb.gimp_image_remove_channel(image,channel_D3a)
	
	if nLayer ==0:
		pdb.gimp_image_remove_channel(image,channel_D4a)	
	
	#create D3 from D4a - D5a															#6 layers (option 1)
	if nLayer >=1:
		channel_D3 = pdb.gimp_channel_copy(channel_D4a)
		pdb.gimp_image_insert_channel(image,channel_D3,None,0)
		pdb.gimp_item_set_name(channel_D3,"D3")
		pdb.gimp_channel_combine_masks(channel_D3,channel_D5a,CHANNEL_OP_SUBTRACT,0,0)
		pdb.gimp_image_remove_channel(image,channel_D4a)
		
	if nLayer ==1:
		pdb.gimp_image_remove_channel(image,channel_D5a)	
	
	#create D4 from D5a - D6a															#8 layers (option 2)
	if nLayer >=2:
		channel_D4 = pdb.gimp_channel_copy(channel_D5a)
		pdb.gimp_image_insert_channel(image,channel_D4,None,0)
		pdb.gimp_item_set_name(channel_D4,"D4")
		pdb.gimp_channel_combine_masks(channel_D4,channel_D6a,CHANNEL_OP_SUBTRACT,0,0)
		pdb.gimp_image_remove_channel(image,channel_D5a)
		pdb.gimp_image_remove_channel(image,channel_D6a)
	
	#create L1 from L2a - L3a
	channel_L1 = pdb.gimp_channel_copy(channel_L2a)
	pdb.gimp_image_insert_channel(image,channel_L1,None,0)
	pdb.gimp_item_set_name(channel_L1,"L1")
	pdb.gimp_channel_combine_masks(channel_L1,channel_L3a,CHANNEL_OP_SUBTRACT,0,0)
	pdb.gimp_image_remove_channel(image,channel_L2a)

	#create L2 from L3a - L4a															#4 layers (option 0)
	channel_L2 = pdb.gimp_channel_copy(channel_L3a)
	pdb.gimp_image_insert_channel(image,channel_L2,None,0)
	pdb.gimp_item_set_name(channel_L2,"L2")
	pdb.gimp_channel_combine_masks(channel_L2,channel_L4a,CHANNEL_OP_SUBTRACT,0,0)
	pdb.gimp_image_remove_channel(image,channel_L3a)
	
	if nLayer ==0:
		pdb.gimp_image_remove_channel(image,channel_L4a)
	
	#create L3 from L4a - L5a															#6 layers (option 1)
	if nLayer >=1:
		channel_L3 = pdb.gimp_channel_copy(channel_L4a)
		pdb.gimp_image_insert_channel(image,channel_L3,None,0)
		pdb.gimp_item_set_name(channel_L3,"L3")
		pdb.gimp_channel_combine_masks(channel_L3,channel_L5a,CHANNEL_OP_SUBTRACT,0,0)
		pdb.gimp_image_remove_channel(image,channel_L4a)
		
	if nLayer ==1:
		pdb.gimp_image_remove_channel(image,channel_L5a)
	
	#create L4 from L5a - L6a															#8 layers (option 2)
	if nLayer >=2:
		channel_L4 = pdb.gimp_channel_copy(channel_L5a)
		pdb.gimp_image_insert_channel(image,channel_L4,None,0)
		pdb.gimp_item_set_name(channel_L4,"L4")
		pdb.gimp_channel_combine_masks(channel_L4,channel_L6a,CHANNEL_OP_SUBTRACT,0,0)
		pdb.gimp_image_remove_channel(image,channel_L5a)
		pdb.gimp_image_remove_channel(image,channel_L6a)
	
	pdb.gimp_selection_none(image)
	pdb.gimp_image_remove_layer(image,layer_copy)										#remove equalized layer that was used to create masks
	
	#create layer with D5 as mask														#10 layers (option 3)
	if nLayer ==3:
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,channel_D5)
		layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
		pdb.gimp_image_insert_layer(image,layer_copy,None,0)
		pdb.gimp_item_set_name(layer_copy,"D5")
		pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
		mask = pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
		pdb.gimp_layer_add_mask(layer_copy,mask)
		pdb.gimp_image_remove_channel(image,channel_D5)

	#create layer with D4 as mask														#8 layers (option 2)
	if nLayer >=2:
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,channel_D4)
		layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
		pdb.gimp_image_insert_layer(image,layer_copy,None,0)
		pdb.gimp_item_set_name(layer_copy,"D4")
		pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
		mask = pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
		pdb.gimp_layer_add_mask(layer_copy,mask)
		pdb.gimp_image_remove_channel(image,channel_D4)

	#create layer with D3 as mask														#6 layers (option 1)
	if nLayer >=1:
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,channel_D3)
		layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
		pdb.gimp_image_insert_layer(image,layer_copy,None,0)
		pdb.gimp_item_set_name(layer_copy,"D3")
		pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
		mask = pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
		pdb.gimp_layer_add_mask(layer_copy,mask)
		pdb.gimp_image_remove_channel(image,channel_D3)

	#create layer with D2 as mask														#4 layers (option 0)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,channel_D2)
	layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,layer_copy,None,0)
	pdb.gimp_item_set_name(layer_copy,"D2")
	pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
	mask = pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
	pdb.gimp_layer_add_mask(layer_copy,mask)
	pdb.gimp_image_remove_channel(image,channel_D2)

	#create layer with D1 as mask
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,channel_D1)
	layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,layer_copy,None,0)
	pdb.gimp_item_set_name(layer_copy,"D1")
	pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
	mask = pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
	pdb.gimp_layer_add_mask(layer_copy,mask)
	pdb.gimp_image_remove_channel(image,channel_D1)
	
	#create layer with L1 as mask
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,channel_L1)
	layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,layer_copy,None,0)
	pdb.gimp_item_set_name(layer_copy,"L1")
	pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
	mask = pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
	pdb.gimp_layer_add_mask(layer_copy,mask)
	pdb.gimp_image_remove_channel(image,channel_L1)

	#create layer with L2 as mask														#4 layers (option 0)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,channel_L2)
	layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,layer_copy,None,0)
	pdb.gimp_item_set_name(layer_copy,"L2")
	pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
	mask = pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
	pdb.gimp_layer_add_mask(layer_copy,mask)
	pdb.gimp_image_remove_channel(image,channel_L2)

	#create layer with L3 as mask														#6 layers (option 1)
	if nLayer >=1:
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,channel_L3)
		layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
		pdb.gimp_image_insert_layer(image,layer_copy,None,0)
		pdb.gimp_item_set_name(layer_copy,"L3")
		pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
		mask = pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
		pdb.gimp_layer_add_mask(layer_copy,mask)
		pdb.gimp_image_remove_channel(image,channel_L3)
	
	#create layer with L4 as mask														#8 layers (option 2)
	if nLayer >=2:
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,channel_L4)
		layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
		pdb.gimp_image_insert_layer(image,layer_copy,None,0)
		pdb.gimp_item_set_name(layer_copy,"L4")
		pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
		mask = pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
		pdb.gimp_layer_add_mask(layer_copy,mask)
		pdb.gimp_image_remove_channel(image,channel_L4)
	
	#create layer with L5 as mask														#10 layers (option 3)
	if nLayer ==3:
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,channel_L5)
		layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
		pdb.gimp_image_insert_layer(image,layer_copy,None,0)
		pdb.gimp_item_set_name(layer_copy,"L5")
		pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
		mask = pdb.gimp_layer_create_mask(layer_copy,ADD_MASK_SELECTION)
		pdb.gimp_layer_add_mask(layer_copy,mask)
		pdb.gimp_image_remove_channel(image,channel_L5)

	pdb.gimp_selection_none(image)
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()

register(
	"python_fu_lum_mask_dm",
	"Create Luminosity Masks",
	"Create Luminosity Masks",
	"David Marsden",
	"David Marsden",
	"December 2019",
	"<Image>/Python-Fu/Luminosity Masks DM...",             							#Menu path
	"RGB*, GRAY*", 
	[
		(PF_OPTION,"nLayer","No. of Masked Layers",2,(" 4"," 6"," 8","10"))
	],
	[],
	python_lum_mask_dm)

main()
