#!/usr/bin/env python

# mask-if.py
# Originated from this request: https://gimplearn.net/viewtopic.php?f=18&t=2921
# So after watching a Photoshop Blend-If Tutorial, I thought I would make this script that masks the layer using Minimum Maximum values
# Using a (Grayscale) Mask dependent upon color values of the layer.

# Created by Tin Tran https://gimplearn.net
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: fix spinner bug (to allow adjustments)
# Rel 3: Speed up version (super fast compared to version 1 & 2)
# Rel 4: Preprocess 0-255 values for (friendliness and reduce overhead processing)
# Rel 5: Even more speed enhancement
# Rel 6: Even a little more speed enhancement
# Rel 7: Undo in one step (Ctrl+Z) so that user can quickly repeat process with other settings
# Rel 8: Less Progress Updates for more speed
# Rel 9: Removed uneeded type casting for more speed
#import Image
from gimpfu import *
import random
import math
from array import array #needed for array pixel access
import time

#import os
def python_mask_if_tt(image, layer, channel, min_lo, min_hi, max_lo, max_hi):
	pdb.gimp_image_undo_group_start(image)
	#copy all layers to layers
	work_layer = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,'mask-if',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,work_layer,None,0)

	width = layer.width
	height = layer.height
	srcRgn = layer.get_pixel_rgn(0, 0, width, height, False, False) #read only
	dstRgn = work_layer.get_pixel_rgn(0, 0, width, height, True, True) #update to pixels
	#x = 0; y = 0;
	p_size_source = len(srcRgn[0,0])
	p_size_dest = len(dstRgn[0,0])
	src_pixels = array("B", srcRgn[0:width, 0:height]) #source
	dest_pixels = array("B", dstRgn[0:width, 0:height]) #destination pixel
	#maskvalue = [0,0,0,255]
	#preprocess all 256 values for speed
	
	pdb.gimp_progress_update(float(0)/255) #update progress
	pdb.gimp_progress_set_text("Preprocess 0-255 values");
	min_lo = int(min_lo)
	min_hi = int(min_hi)
	max_lo = int(max_lo)
	max_hi = int(max_hi)
	#start = time.time()
	preprocess_values = range(0,256)
	for i in range(0,min_lo): #first chunk below min_lo
		preprocess_values[i] = array("B",[0,0,0,255])
		#preprocess_values.append(array("B",[0,0,0,255]))
	pdb.gimp_progress_update(float(min_lo)/255) #update progress	
	for i in range(min_lo,min_hi+1):
		thisvalue = i
		g_value = int(float((thisvalue - min_lo)) / max((min_hi-min_lo),1) * 255)
		preprocess_values[i] = array("B",[g_value,g_value,g_value,255])
	pdb.gimp_progress_update(float(min_hi+1)/255) #update progress	
	for i in range(min_hi+1,max_lo):
		preprocess_values[i] = array("B",[255,255,255,255])
	pdb.gimp_progress_update(float(max_lo)/255) #update progress	
	for i in range(max_lo,max_hi+1):
		thisvalue = i
		g_value = int(255 - float((thisvalue - max_lo)) / max((max_hi-max_lo),1) * 255)
		preprocess_values[i] = array("B",[g_value,g_value,g_value,255])
	pdb.gimp_progress_update(float(max_hi+1)/255) #update progress	
	for i in range(max_hi+1,256):
		preprocess_values[i] = array("B",[0,0,0,255])
	pdb.gimp_progress_update(float(255)/255) #update progress	
	#end = time.time()
	#pdb.gimp_message(end-start)
	'''	#Do above instead of below to dodge multiple ifs for slight speed
	for i in range(0,256):
		pdb.gimp_progress_update(float(i)/255) #update progress
		thisvalue = i
		if (thisvalue < min_lo) or (thisvalue > max_hi): #outside of range make it completely black
			preprocess_values.append(array("B",[0,0,0,255]))
		elif (thisvalue > min_hi) and (thisvalue < max_lo):
			preprocess_values.append(array("B",[255,255,255,255]))
		elif (thisvalue >= min_lo) and (thisvalue <= min_hi):
			g_value = int((thisvalue - min_lo) / max((min_hi-min_lo),1) * 255)
			preprocess_values.append(array("B",[g_value,g_value,g_value,255]))
		elif (thisvalue >= max_lo) and (thisvalue <= max_hi):
			g_value = int(255 - (thisvalue - max_lo) / max((max_hi-max_lo),1) * 255)
			preprocess_values.append(array("B",[g_value,g_value,g_value,255]))
	'''		
	pdb.gimp_progress_set_text("Mask-If Processing");
	#Code below are identical except for how thisvalue set is dependent upon channel for speed.
	if (channel == 0):
		src_pos = 0
		dest_pos = 0
		total = height * width
		progress_each = total/10
		showcount = 0
		#blank = map(translatePixel,range(0,total))
		#start = time.time()
		for y in range(0,total):
			showcount += 1
			if (showcount == progress_each): 
				pdb.gimp_progress_update(float(y)/total)
				showcount = 0
			pixel = src_pixels[src_pos: src_pos + p_size_source]
			dest_pixels[dest_pos : dest_pos + p_size_dest] = preprocess_values[(pixel[0] + pixel[1] + pixel[2])/3]
			src_pos += p_size_source
			dest_pos += p_size_dest
		#end = time.time()
		#pdb.gimp_message(end-start)
	if (channel == 1): #red value
		src_pos = 0
		dest_pos = 0
		total = height * width
		progress_each = total/10
		showcount = 0
		for y in range(0,total):
			showcount += 1
			if (showcount == progress_each): 
				pdb.gimp_progress_update(float(y)/total)
				showcount = 0
			#for x in range(0,width):
			pixel = src_pixels[src_pos: src_pos + p_size_source]
			dest_pixels[dest_pos : dest_pos + p_size_dest] = preprocess_values[pixel[0]]
			src_pos += p_size_source
			dest_pos += p_size_dest
	if (channel == 2):
		src_pos = 0
		dest_pos = 0
		total = height * width
		progress_each = total/10
		showcount = 0
		for y in range(0,total):
			showcount += 1
			if (showcount == progress_each): 
				pdb.gimp_progress_update(float(y)/total)
				showcount = 0
			#for x in range(0,width):
			pixel = src_pixels[src_pos: src_pos + p_size_source]
			dest_pixels[dest_pos : dest_pos + p_size_dest] = preprocess_values[pixel[1]]
			src_pos += p_size_source
			dest_pos += p_size_dest
	if (channel == 3):
		src_pos = 0
		dest_pos = 0
		total = height * width
		progress_each = total/10
		showcount = 0
		for y in range(0,total):
			showcount += 1
			if (showcount == progress_each): 
				pdb.gimp_progress_update(float(y)/total)
				showcount = 0
			#for x in range(0,width):
			pixel = src_pixels[src_pos: src_pos + p_size_source]
			dest_pixels[dest_pos : dest_pos + p_size_dest] = preprocess_values[pixel[2]]
			src_pos += p_size_source
			dest_pos += p_size_dest

	#call these functions to update drawable with pixel changes, mass update to set_pixel/put_pixel calls
	dstRgn[0:width, 0:height] = dest_pixels.tostring() 	
	work_layer.flush()
	work_layer.merge_shadow(True)
	work_layer.update(0, 0, width, height)
	pdb.gimp_displays_flush()
		
	mask = pdb.gimp_layer_create_mask(layer,ADD_MASK_WHITE)
	pdb.gimp_layer_add_mask(layer,mask)
	pdb.gimp_selection_all(image)
	pdb.gimp_edit_copy(work_layer)
	floating_sel = pdb.gimp_edit_paste(mask,TRUE)
	pdb.gimp_floating_sel_anchor(floating_sel);
	pdb.gimp_item_set_visible(work_layer,FALSE)
	pdb.gimp_selection_none(image)	
	#pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_fu_mask_if_tt",
	"Mask If Like Photoshop's Blend If",
	"Mask If Like Photoshop's Blend If",
	"Tin Tran",
	"Tin Tran",
	"Feb 2021",
	"<Image>/Python-Fu/A Mask-If...",             #Menu path
	"RGB*, GRAY*, INDEXED*", 
	[
	(PF_OPTION,"channel",   "Channel:", 0, ["Grey Value","Red Value","Green Value","Blue Value"]), # initially 0th is choice
	(PF_SPINNER, "min_lo", "Minimum Low:", 50, (0, 255, 1)),  # alias PF_ADJUSTMENT
	(PF_SPINNER, "min_high", "Minimum High:", 100, (0, 255, 1)),  # alias PF_ADJUSTMENT
	(PF_SPINNER, "max_lo", "Maximum Low:", 150, (0, 255, 1)),  # alias PF_ADJUSTMENT
	(PF_SPINNER, "max_high", "Maximum High:", 200, (0, 255, 1)),  # alias PF_ADJUSTMENT
	],
	[],
	python_mask_if_tt)

main()
