#!/usr/bin/env python
# coding=utf-8
# -*- encoding: utf-8 -*-
# paint_based_on_paths_2.py Rel 7
# Created by Tin Tran
# Comments directed to http://gimplearn.net
# Creates new image, paints random strokes that has the same angle as closest points on paths
# defined.
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# ======================== AUTOCOMPLETE HELP STARTS =============================
import random
from gimpfu import *
import math
# DMC = [["1","Blue Green","#199ebd"],
# ["2","Black","#232323"],
# ["3","Blue Violet","#7366bd"],
# ["4","Violet(Purple)","#926eae"],
# ["5","Green","#1cac78"],
# ["6","Blue","#1f75fe"],
# ["7","Yellow Green","#c5e384"],
# ["8","White","#ededed"],
# ["9","Yellow","#fce883"],
# ["10","Carnation Pink","#ffaacc"],
# ["11","Yellow Orange","#ffb653"],
# ["12","Red","#ee204d"],
# ["13","Brown","#b5674d"],
# ["14","Orange","#ff7538"],
# ["15","Red Orange","#ff5349"],
# ["16","Red Violet","#c0448f"]];
DMC = [["1","Blue Green","#199ebd"],
["2","Black","#232323"],
["3","Blue Violet","#7366bd"],
["4","Violet(Purple)","#926eae"],
["5","Green","#1cac78"],
["6","Blue","#1f75fe"],
["7","Yellow Green","#c5e384"],
["8","White","#ededed"],
["9","Yellow","#fce883"],
["10","Carnation Pink","#ffaacc"],
["11","Yellow Orange","#ffb653"],
["12","Red","#ee204d"],
["13","Brown","#b5674d"],
["14","Orange","#ff7538"],
["15","Red Orange","#ff5349"],
["16","Red Violet","#c0448f"],
["17","Gray","#95918c"],
["18","Cerulean","#1dacd6"],
["19","Indigo","#5d76cb"],
["20","Green Yellow","#f0e891"],
["21","Dandelion","#fddb6d"],
["22","Apricot","#fdd9b5"],
["23","Violet Red","#f75394"],
["24","Scarlet","#fc2847"]
];
def hex_to_rgb(hexa):
	hexa = hexa[-6:]
	t = tuple(int(hexa[i:i+2], 16)  for i in (0, 2, 4))
	return [t[0],t[1],t[2]]
for i in range(0,len(DMC)):
	DMC[i].append(hex_to_rgb(DMC[i][2])); #add RGB value onto the end.
#Copied this from ofn-text-along-path.py returns angle in radians
def computeOrientedSlope(dx,dy,slope): #This is a better version than below computeThetaWithSlope function.
	if abs(slope) > 100000: # very vertical
		return math.atan2(dy,dx) # no perfect, but properly oriented
		# keep dx/dy signs but give then the same ratio as in slope
	return math.atan2(math.copysign(slope,dy),math.copysign(1,dx))
#just returns distance square as we don't care to get distance but just degree to compare them
def distance(p0, p1):
	return ((p0[0] - p1[0])**2 + (p0[1] - p1[1])**2)
#reads points and angles at each point into points array
def read_points_angles(image,layer,spacing,offset_angle):
	points = []
	if len(image.vectors) < 1:
		pdb.gimp_message("You must define a path/vector")
		raise Exception('You must define a path/vector')
	vectors = image.vectors[0]
	strokes = vectors.strokes
	for stroke in strokes: #loop through all strokes
		stroke_length = stroke.get_length(0.01) - 1.1 #subtract 1.1 so we can get the point at +1 and its' still valid
		sections = int(stroke_length/spacing)
		for i in range(0,sections+1): #loop from 0 to sections
			dist1 = i * spacing
			dist2 = dist1 + 1 #1
			x1,y1,slope1,valid1 = stroke.get_point_at_dist(dist1,0.01)
			x2,y2,slope2,valid2 = stroke.get_point_at_dist(dist2,0.01)
			rad_angle = computeOrientedSlope(x2-x1, y2-y1, slope1)
			angle = math.degrees(rad_angle)
			angle = (angle + offset_angle + random.randrange(0,30) - 15) % 360
			if (angle > 180): #make it fit between -180 and 180
				angle = -180 + (angle-180)
			points.append([[x1,y1],angle])
			#pdb.gimp_message(str(points[len(points)-1]))
	return points
#main method
def python_paint_based_on_paths_2_tt(image, layer, brush, brush_size, brush_spacing, offset_angle, random_angle):
	#set up colors
	colors = []
	inrange = 255;
	pdb.gimp_message("adding colors...");
	for one in range(0,len(DMC)):
		for two in range(one,len(DMC)):
			for three in range(two,len(DMC)):
				for four in range(three,len(DMC)):
					for five in range(four,len(DMC)):
						if (one != two or two != three or one != three or one!= four or two!=four or three!=four or one!=five or two!=five or three!=five or four!=five):
							one_ = DMC[one][3];
							two_ = DMC[two][3];
							three_ = DMC[three][3];
							four_ = DMC[four][3];
							five_ = DMC[five][3];
							minR = min(one_[0],two_[0],three_[0],four_[0],five_[0]);maxR = max(one_[0],two_[0],three_[0],four_[0],five_[0]);
							minG = min(one_[1],two_[1],three_[1],four_[1],five_[1]);maxG = max(one_[1],two_[1],three_[1],four_[1],five_[1]);
							minB = min(one_[2],two_[2],three_[2],four_[2],five_[2]);maxB = max(one_[2],two_[2],three_[2],four_[2],five_[2]);
							#if (((maxR-minR)<=inrange) and ((maxG-minG)<=inrange) and ((maxB-minB)<=inrange)):
							average_ = [((one_[0]+two_[0]+three_[0]+four_[0]+five_[0])/5.0),((one_[1]+two_[1]+three_[1]+four_[1]+five_[1])/5.0),((one_[2]+two_[2]+three_[2]+four_[2]+five_[2])/5.0)]
							diff = 0;
							colors.append([one_,two_,three_,four_,five_,average_,diff]);
	pdb.gimp_message(str(len(colors)) + " colors")
	draw_angle = 0.0;
	#moved this up top so that we can see new image right away before we do anything to record things
	new_image = pdb.gimp_image_new(layer.width, layer.height, RGB)
	pdb.gimp_display_new(new_image)
	source_layer = pdb.gimp_layer_new_from_drawable(layer,new_image)
	pdb.gimp_image_insert_layer(new_image, source_layer, None, 0)
	pdb.gimp_layer_set_name(source_layer, "Source")
	paint_layer = pdb.gimp_layer_new(new_image, layer.width, layer.height, RGBA_IMAGE, "Paint", 100, NORMAL_MODE)
	pdb.gimp_image_insert_layer(new_image, paint_layer, None, 0)


	#pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()

	if random_angle == 0:
		points = read_points_angles(image,layer,brush_spacing,offset_angle)
	else:
		points = []
	draw_points = []
	y_sects = int(layer.height/brush_spacing)
	x_sects = int(layer.width/brush_spacing)
	for iy in range(0,y_sects+1):
		for ix in range(0,x_sects+1):
			x = ix * brush_spacing
			y = iy * brush_spacing
			draw_points.append([x,y])
	#sort it random order so that it's shuffled good
	draw_points.sort(key=lambda x:random.randrange(0,layer.width*layer.height))
	pdb.gimp_message("Randomization of draw locations completed.")
	#pdb.gimp_message(str(draw_points))

	

	#work_layer = pdb.gimp_layer_new(new_image, layer.width, layer.height, RGBA_IMAGE, "Work", 100, LAYER_MODE_NORMAL)
	#pdb.gimp_image_insert_layer(new_image, work_layer, None, 0)

	pdb.gimp_context_set_brush(brush)
	#pdb.gimp_context_set_brush("Van Gogh 20 (ani)")
	pdb.gimp_context_set_brush_size(brush_size)
	strokes_counter = 0;
	for dp in range(0,len(draw_points)):
		draw_point = draw_points[dp]
		shortestdist = 10000000
		if random_angle == 0:
			for p in range(0,len(points)):
				point = points[p]
				dist = distance(draw_point, point[0])
				if dist < shortestdist:
					shortestdist = dist
					draw_angle = point[1] #grab angle
					
					
		else:
			draw_angle = random.randrange(0,360)-180;
		oversize = 0;
		# if (shortestdist <= brush_size):
		# 	#if it's closer to brushsize, use brushsize
		# 	pdb.gimp_context_set_brush_size(brush_size)
		# else:
		# 	#else use shortestdist as brushsize or 3 times brushsize(max)
		# 	pdb.gimp_context_set_brush_size(min(brush_size*10,brush_size+(shortestdist-brush_size)/7))
		#	oversize = 1;
		maxsize = 190;
		shortestdist = shortestdist**0.5; #not squarerooted yet so we do it here
		bsize = min(maxsize,brush_size+float(shortestdist)*3.0*(maxsize-brush_size)/maxsize)
		pdb.gimp_context_set_brush_size(bsize)

		pdb.gimp_context_set_brush_angle(draw_angle)

		#pdb.gimp_drawable_edit_clear(work_layer)
		#pdb.gimp_paintbrush(work_layer, 0, 2, draw_point, PAINT_CONSTANT, 0)
		#pdb.gimp_image_select_item(new_image, CHANNEL_OP_REPLACE, work_layer)
		#pdb.gimp_drawable_edit_clear(work_layer)
		
		#non_empty,x1,y2,x2,y2 = pdb.gimp_selection_bounds(new_image)
		#if non_empty==TRUE: #if there is a selection
			# r, _, _, _, _, _ = pdb.gimp_histogram(source_layer,HISTOGRAM_RED,0,255)
			# g, _, _, _, _, _ = pdb.gimp_histogram(source_layer,HISTOGRAM_GREEN,0,255)
			# b, _, _, _, _, _ = pdb.gimp_histogram(source_layer,HISTOGRAM_BLUE,0,255)
			# pdb.gimp_context_set_foreground((int(r),int(g),int(b)))
		# if (draw_point[0]+1) >= layer.width:
		# 	offsetx = -1
		# else:
		# 	offsetx = 1
		# if (draw_point[1]+1) >= layer.height:
		# 	offsety = -1
		# else:
		# 	offsety = 1
		draw_point = [int(draw_point[0]),int(draw_point[1])]
		#if 1==1:
		#if oversize == 1: #if it's oversize brush, make sure it's not painted yet so that small defined brush size gets to do detailing later on
			#pdb.gimp_image_select_item(new_image,CHANNEL_OP_REPLACE,paint_layer) #select painted area
			#sel_value = pdb.gimp_selection_value(new_image,draw_point[0],draw_point[1])
			#pdb.gimp_selection_none(new_image)
			#if (sel_value == 0): #not yet selected so not painted
			#GET PIXEL CHECK SHOULD BE FASTER
		if (oversize == 1):
			paint_it = 0;
			try:	
				pixel = paint_layer.get_pixel(draw_point[0],draw_point[1])
				if pixel[3] < 128:
					paint_it = 1;
			except Exception, e:
				#pdb.gimp_message("Errored:" + str(e) + "...continuing")
				pass
		else:
			paint_it = 1;	
		if (paint_it == 1):
			try:
				pixel1 = source_layer.get_pixel(draw_point[0], draw_point[1])
				# ================ Try different tints/shades
				R = int(pixel1[0])
				G = int(pixel1[1])
				B = int(pixel1[2])
				color_error = 12 #percent
				value_change = random.randrange(0,int(color_error)*10)/10.0 - color_error/2.0 #value change
				if value_change < 0: #darker shade
					R = int(R*(1.0+value_change/100.0)) #-10 will make it multiplied by .90 or 90%
					G = int(G*(1.0+value_change/100.0))
					B = int(B*(1.0+value_change/100.0))
				elif value_change > 0:
					R = int(R+(255.0-R)*(value_change/100.0)); #10 will make it 10 percent of distance to 255
					G = int(G+(255.0-G)*(value_change/100.0)); #10 will make it 10 percent of distance to 255
					B = int(B+(255.0-B)*(value_change/100.0)); #10 will make it 10 percent of distance to 255
				#pdb.gimp_context_set_foreground((R,G,B))
				for c in range(0,len(colors)):
					color = colors[c][5];
					#d = sqrt(((r2-r1)*0.3)^2 + ((g2-g1)*0.59)^2 + ((b2-b1)*0.11)^2)
					colors[c][6] = ((R-color[0])*0.3)**2 + ((G-color[1])*0.59)**2 + ((B-color[2])*0.11)**2
				colors =  sorted(colors, key = lambda x:x[6])
				chosencolors = colors[0];
				colorI = random.randrange(0,5);
				C = chosencolors[colorI];
				pdb.gimp_context_set_foreground((C[0],C[1],C[2]))
			#pdb.gimp_context_set_foreground(((pixel1[0]+pixel2[0]+pixel3[0]+pixel4[0])/4,(pixel1[1]+pixel2[1]+pixel3[1]+pixel4[1])/4,(pixel1[2]+pixel2[2]+pixel3[2]+pixel4[2])/4))
			#pdb.gimp_paintbrush(paint_layer, 0, 2, draw_point, PAINT_CONSTANT, 0)
				pdb.gimp_paintbrush_default(paint_layer, 2, draw_point)
				strokes_counter += 1;
			#pdb.gimp_message(str(draw_point))
			except Exception, e:
				#pdb.gimp_message("Errored:" + str(e) + "...continuing")
				pass
		#pdb.gimp_edit_fill(paint_layer,FOREGROUND_FILL)
		#pdb.gimp_selection_none(new_image) #select none before we draw.
			#pdb.gimp_displays_flush()
		if dp % 100 == 0:
			pdb.gimp_displays_flush()
		if dp % 1000==0:
			pdb.gimp_message("Strokes: "+str(strokes_counter))
			
	pdb.gimp_context_pop()
	#pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	pdb.gimp_message("Complete!")
	#return

register(
	"python_fu_paint_based_on_paths_2_tt",
	"Creates new image, paints random strokes with angle same as closest path points",
	"Creates new image, paints random strokes with angle same as closest path points",
	"Tin Tran",
	"Tin Tran",
	"September 2018",
	"<Image>/Python-Fu/Paint Based on Paths 2...",             #Menu path
	"RGB*, GRAY*",
	[
	(PF_BRUSH, "brush", "Brush:", 0),
	(PF_SPINNER, "brush_size", "Brush-size:", 25, (1, 1000, 1)),
	(PF_SPINNER, "brush_spacing", "Brush-spacing:", 3, (1, 1000, 1)),
	(PF_SPINNER, "offset_angle", "Offset angle (angle to offset from path's angles):", 0, (0,360, 1)),
	(PF_TOGGLE, "random_angle",   "Use random angle (when doing generally round blotches):", 0),
	#(PF_SPINNER, "px_height", "Height of each tile image:", 100, (1, 1000, 50)),
	],
	[],
	python_paint_based_on_paths_2_tt)

main()
