#!/usr/bin/env python

# paint_paths_in_steps.py Rel 2
# Created by Tin Tran
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
# Rel 2: Follow path closer to pixel accuracy
from gimpfu import *
import random
def python_tt_paint_paths_in_steps(image, layer, steps) : #FUNCTION DEFINITION
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#creates a working layer
	work_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,'work layer',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,work_layer,None,0)

	#new image used for animation
	new_image = pdb.gimp_image_new(image.width,image.height,RGB)
	new_display = pdb.gimp_display_new(new_image)

	#get active vector and work with it
	vectors = pdb.gimp_image_get_active_vectors(image)
	num_strokes, stroke_ids = pdb.gimp_vectors_get_strokes(vectors)

	#get total stroke length
	total_length = 0
	for i in range(0,num_strokes):
		stroke_id = stroke_ids[i] #current stroke
		current_length = pdb.gimp_vectors_stroke_get_length(vectors,stroke_id,0.01)
		total_length += current_length

	step_length = float(total_length)/steps #length per step
	for i in range(0,num_strokes):
		stroke_id = stroke_ids[i] #current stroke
		current_length = pdb.gimp_vectors_stroke_get_length(vectors,stroke_id,0.01)
		current_at = 0.01
		x,y,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,current_at,0.01)
		oldx,oldy = x,y
		while current_at < current_length-0.01:
			#x,y,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,current_at,0.01)	
			#pdb.gimp_paintbrush_default(work_layer,4,[oldx,oldy,x,y]) #paint to new point
			next_at = (current_at + step_length)
			if next_at > current_length-0.01:
				next_at = current_length-0.01
			this_at = current_at
			while this_at < next_at:
				nox,noy,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,this_at,0.01)	
				this_at += 1
				if this_at > current_length-0.01:
					this_at = current_length-0.01
				nx,ny,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,this_at,0.01)	
				pdb.gimp_paintbrush_default(work_layer,4,[nox,noy,nx,ny])
			#pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
			oldx,oldy = x,y
			current_at += step_length
			new_layer = pdb.gimp_layer_new_from_visible(image,new_image,"painted")
			pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
	
		#x,y,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,current_length-0.01,0.01)	
		#pdb.gimp_paintbrush_default(work_layer,4,[oldx,oldy,x,y]) #paint to final point
		#new_layer = pdb.gimp_layer_new_from_visible(image,new_image,"painted")
		

	#YOUR CODE ENDS ========================
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	#return

register(
	"python_fu_tt_paint_paths_in_steps",
	"Paint the paths in steps",
	"Paint the paths in steps",
	"Tin Tran",
	"Tin Tran",
	"June 2019",
	"<Image>/Python-Fu/Paint Paths in Steps...",
	"*",      # Create a new image, don't work on an existing one
	[ 
	#INPUT
	(PF_SPINNER, "steps", "frames:", 50, (10,5000,1)), 
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_blur_radius", "Shadow Blur Radius:", 15, (0,1024,1)),
	#(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 100, (0,100,1)),
	#INPUT ENDS
	],
	[],
	python_tt_paint_paths_in_steps)

main()