#!/usr/bin/env python

# paper_cutout.py Rel 1
# Created by Tin Tran https://gimplearn.net
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
import math
import string
#import Image
from gimpfu import *
from array import array
import sys
def python_paper_cutout(image, layer, texture, background, foreground,blurradius) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()


	pdb.gimp_selection_none(image)

	#loads texture layer
	textureimage = pdb.gimp_file_load(texture,texture)
	texturelayer = pdb.gimp_image_get_active_layer(textureimage)
	texturecopy = pdb.gimp_layer_new_from_drawable(texturelayer,image)
	pdb.gimp_image_insert_layer(image,texturecopy,None,0);
	pdb.gimp_image_delete(textureimage)

	#desaturate texture layer
	pdb.gimp_drawable_desaturate(texturecopy, DESATURATE_LUMINANCE)

	#add a fill color multiply layer
	bgcolorlayer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,'multiply bgcolor',100,LAYER_MODE_MULTIPLY)
	pdb.gimp_image_insert_layer(image,bgcolorlayer,None,0)
	pdb.gimp_context_set_foreground(background)
	pdb.gimp_drawable_edit_fill(bgcolorlayer,FILL_FOREGROUND)

	#add an overlay on top of background
	bgoverlayer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,'bg overlay',80,LAYER_MODE_OVERLAY)
	pdb.gimp_image_insert_layer(image,bgoverlayer,None,0)
	pdb.gimp_context_set_default_colors()
	#pdb.gimp_context_set_gradient_fg_bg_hsv_cw()
	pdb.gimp_edit_blend(bgoverlayer,BLEND_FG_BG_RGB,LAYER_MODE_NORMAL,GRADIENT_LINEAR,100,0.0,REPEAT_NONE,FALSE,FALSE,3,0.2,FALSE,0,image.height,image.width,0)

	#selects our image/text
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	#creates paper texture of our image/text
	imagetexturecopy = pdb.gimp_layer_new_from_drawable(texturecopy,image)
	pdb.gimp_image_insert_layer(image,imagetexturecopy,None,0)
	pdb.gimp_layer_add_alpha(imagetexturecopy)
	pdb.gimp_selection_invert(image)
	pdb.gimp_edit_clear(imagetexturecopy)

	pdb.gimp_selection_none(image);
	#add a fill color multiply layer
	fgcolorlayer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,'multiply bgcolor',100,LAYER_MODE_MULTIPLY)
	pdb.gimp_image_insert_layer(image,fgcolorlayer,None,0)
	pdb.gimp_context_set_foreground(foreground)
	pdb.gimp_drawable_edit_fill(fgcolorlayer,FILL_FOREGROUND)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	pdb.gimp_selection_invert(image)
	pdb.gimp_edit_clear(imagetexturecopy)

	#merge down multiply and texture of text/image
	textimagelayer = pdb.gimp_image_merge_down(image,fgcolorlayer,EXPAND_AS_NECESSARY)
	pdb.gimp_selection_invert(image)

	#overlay of text/image
	fgoverlayer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,'fg overlay',80,LAYER_MODE_OVERLAY)
	pdb.gimp_image_insert_layer(image,fgoverlayer,None,0)
	pdb.gimp_context_set_default_colors()
	#pdb.gimp_context_set_gradient_fg_bg_hsv_cw()
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	width = x2 - x1
	height = y2 - y1
	new_x1 = x1 - width/2
	new_x2 = x2 + width/2
	new_y1 = y1 - height/2
	new_y2 = y2 + width/2
	x1,y1,x2,y2 = new_x1,new_y1,new_x2,new_y2
	pdb.gimp_edit_blend(fgoverlayer,BLEND_FG_BG_RGB,LAYER_MODE_NORMAL,GRADIENT_LINEAR,100,0.0,REPEAT_NONE,FALSE,FALSE,3,0.2,FALSE,x2,y1,x1,y2)
	pdb.gimp_selection_none(image)

	#inner Shadow layer.
	fginnerlayer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,'fg inner shadow',100,LAYER_MODE_GRAIN_MERGE)
	pdb.gimp_image_insert_layer(image,fginnerlayer,None,0)
	pdb.gimp_edit_fill(fginnerlayer,FILL_FOREGROUND)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	pdb.gimp_edit_clear(fginnerlayer)
	pdb.gimp_layer_translate(fginnerlayer,-blurradius/3.0,+blurradius/3.0)
	pdb.gimp_selection_none(image)
	pdb.plug_in_gauss(image,fginnerlayer,blurradius,blurradius,1)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	pdb.gimp_selection_invert(image)
	pdb.gimp_edit_clear(fginnerlayer)
	pdb.gimp_selection_none(image)
	
	#outer Shadow layer.
	fgshadowlayer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,'fg shadow',100,LAYER_MODE_GRAIN_MERGE)
	pdb.gimp_image_insert_layer(image,fgshadowlayer,None,0)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	pdb.gimp_edit_fill(fgshadowlayer,FILL_FOREGROUND)
	pdb.gimp_layer_translate(fgshadowlayer,-blurradius/3.0,+blurradius/3.0)
	pdb.gimp_selection_none(image)
	pdb.plug_in_gauss(image,fgshadowlayer,blurradius,blurradius,1)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	pdb.gimp_edit_clear(fgshadowlayer)
	
	pdb.gimp_selection_none(image)
	
	





	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_fu_paper_cutout",
	"Creates Paper-cutout effect",
	"Creates Paper-cutout effect",
	"Tin Tran",
	"Tin Tran",
	"October 2019",
	"<Image>/Python-Fu/Paper Cutout...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_FILENAME, "texture", "Paper Texture Image:", 0),
	(PF_COLOR, "background", "Background Color:", (133, 206, 231)),
	(PF_COLOR, "foreground", "Foreground Color:", (255, 117, 129)),
	(PF_SPINNER, "blurradius", "Shadow Blur Radius:", 100, (1, 2000, 10)),
	],
	[],
	python_paper_cutout)

main()
