#!/usr/bin/env python

# prepare_text_into_selection.py Release 2
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Handle paragraph breaks (2 [hard] new-line characters) (TO BE USED with version 10 of Filled Text Selection Plug-in)
from gimpfu import *
import random
#from array import array
#import sys
new_paragraph = "<!--new-paragraph-->"

def find_next_break(t,fontsize,font,width):
	if t.startswith(new_paragraph): #paragraph check
		return new_paragraph,t[len(new_paragraph):]
	start = 0
	space_at = t.find(" ",start)
	if space_at == -1: #if we don't find it, return t as head, "" as tail
		return t,""
	head = t[0:space_at]
	tail = t[space_at+1:]
	if tail.startswith(new_paragraph): #paragraph check
		return head,tail
	_width,_height,_ascent,_descent = pdb.gimp_text_get_extents_fontname(head,fontsize,0,font)
	while True: #keep looking
		#pdb.gimp_message (head + ":" + str(_width) + "<" + str(width))
		start=space_at+1
		space_at = t.find(" ",start)
		if space_at == -1:
			return head,tail
		next_head = t[0:space_at]
		next_tail = t[space_at+1:]
		if next_tail.startswith(new_paragraph): #paragraph check
			return next_head,next_tail
		_next_width,_height,_ascent,_descent = pdb.gimp_text_get_extents_fontname(next_head,fontsize,0,font)	
		if _next_width >= width: #if we passed the width or equal to it check to return closest (head,tail) pair
			cur_diff = abs(width-_width) #distance of head
			next_diff = abs(width-_next_width) #distance of next_head
			if cur_diff <= next_diff:
				return head,tail
			else:
				return next_head,next_tail
		#if we get here means we'll loop again so set these below for next time around.
		_width = _next_width
		head = next_head
		tail = next_tail
	return head,tail #we should never hit this line because of the while True loop:

def textheight(text,fontsize,font,selections):
	t = text.replace("\n\n"," " + new_paragraph + " ")
	t = t.replace("\n"," ").strip() + " " #make it all one-line, we'll break lines ourselves based on selections
	
	#get an initial height
	_width,_height,_ascent,_descent = pdb.gimp_text_get_extents_fontname(t,fontsize,0,font)
	cur_y = int(_height/2.0)
	lines = []
	while (len(t) > 0) and (cur_y < len(selections)): #while there's text left to process and we're still in selection
		#pdb.gimp_message("running find next break")
		head,tail = find_next_break(t,fontsize,font,selections[cur_y][2])
		lines.append(head)
		t = tail
		cur_y += _height
	
	if cur_y >= len(selections): #outside of selection
		process_t = "\n".join(lines)
		_width,process_height,_ascent,_descent = pdb.gimp_text_get_extents_fontname(process_t,fontsize,0,font)
		return process_height + _height,process_t + "\n" + t
	elif len(t) == 0:
		process_t = "\n".join(lines)
		_width,process_height,_ascent,_descent = pdb.gimp_text_get_extents_fontname(process_t,fontsize,0,font)
		return process_height,process_t

def python_tt_prepare_text_into_selection(image, layer):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#core code goes here/below
	if not pdb.gimp_item_is_text_layer(layer):
		pdb.gimp_message("Layer must be a text layer!")
	else: #it's a text layer we're good to go
		non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
		if not non_empty:
			pdb.gimp_message("Must have an active selection")
		else:
			saved_selection = pdb.gimp_selection_save(image)
			sel_width = x2 - x1
			sel_height = y2 - y1
			text = pdb.gimp_text_layer_get_text(layer)
			texts = text.split("\n")
			font = pdb.gimp_text_layer_get_font(layer)
			fontsize,unit = pdb.gimp_text_layer_get_font_size(layer)
			
			#read selection information, don't know if there's a better way right now
			selections = [] #store rows of [x1,x2,x2-x1], index is y (0 offset)
			#pdb.gimp_progress_set_text("Reading Selection Info.")
			pdb.gimp_progress_init("Reading Selection Info.",None)
			for sel_y in range(0,sel_height):
				pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,saved_selection)
				pdb.gimp_image_select_rectangle(image,CHANNEL_OP_INTERSECT,0,y1+sel_y,image.width,1)
				_non_empty,_x1,_y1,_x2,_y2 = pdb.gimp_selection_bounds(image)
				_sel_width = _x2-_x1
				_sel_height = _y2-_y1
				selections.append([_x1,_x2,_sel_width])
				#pdb.gimp_progress_set_text("Reading Selection Info. "+str(int(1.0*sel_y/sel_height*100))+"%")
				pdb.gimp_progress_update(1.0*sel_y/sel_height)


			#using binary search found in below link but with some alteration to seek out our maximum fontsize that
			#would fit in the selection.
			#https://stackoverflow.com/questions/9501337/binary-search-algorithm-in-python
			
			selectionheight = sel_height
			lower = 0 #min font size
			upper = 1024 #max font size
			previous_x = -10
			pdb.gimp_progress_set_text("Searching for optimal fontsize.")
			progress = 0
			while (lower < upper):   # use < instead of <=
				pdb.gimp_progress_update(progress/10.0)

				x = lower + (upper - lower) // 2
				val = x
				if x == previous_x: 
					found = x
					break
				previous_x = x

				#font calculations before binary logic comparison below 
				theight,t = textheight(text,x,font,selections)
				#pdb.gimp_message(str(theight) + ":" + str(sel_height))
				theight_lower,t = textheight(text,lower,font,selections)
				theight_upper,t = textheight(text,upper,font,selections)
				

				if selectionheight == theight:
					found = x
					break
				elif theight < selectionheight:
					if selectionheight == theight_lower:
						found = lower
						break
					lower = x
				elif theight > selectionheight:
					if selectionheight == theight_upper:
						found = upper
						break
					upper = x
				progress += 1.0	
			pdb.gimp_progress_update(1.0)
	
			#here we should have "found" set to optimal fontsize		
			theight,t = textheight(text,found,font,selections)
			#fontname = pdb.gimp_text_fontname(image,None,0,0,t,0,TRUE,found,0,font)
			t = t.replace(new_paragraph," ")
			fontname = pdb.gimp_text_fontname(image,None,0,0,t,0,TRUE,found,0,font)



			

			pdb.gimp_selection_none(image)
			#pdb.gimp_item_transform_scale(merged_result,x1,y1,x2,y2)
			#pdb.gimp_item_set_name(merged_result,"Filled Text Selection")

	#UNCOMMENT BELOW WHEN DONE
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_prepare_text_into_selectionn",                           
	"Prepare Text into Selection to be used with Fill Text Selection plug-in",
	"Prepare Text into Selection to be used with Fill Text Selection plug-in",
	"Tin Tran",
	"Tin Tran",
	"March 2019",
	"<Image>/Python-Fu/Prepare Text Into Selection...",             #Menu path
	"RGB*, GRAY*", 
	[
	#(PF_SPINNER, "frame_percent", "Frame percentage (percent to grow by to create border):", 10, (5, 100, 1)),
	#(PF_TOGGLE, "keep_aspect", "Keep Aspect Ratio of Border", 1),
	#(PF_TOGGLE, "scale_last","Scale Last Line (Sometimes it's too short, you don't want to scale it):", 1),
	],
	[],
	python_tt_prepare_text_into_selection)

main()