#!/usr/bin/env python
# coding=utf-8
# rectangle_to_circle_path.py
# Created by Tin Tran https://gimplearn.net#
# Suggested by Krikor
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html#
#
# ------------
#| Change Log |
# ------------
# 1. - Initial version
from gimpfu import *
import math
def python_rectangle_to_circle_path(image, layer,innerR,outerR):
	#pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	# pdb.script_fu_selection_to_pattern()
	width = int(outerR*2)
	height = int(outerR*2)
	cx = outerR
	cy = outerR
	new_image = pdb.gimp_image_new(width,height,RGB)
	pdb.gimp_display_new(new_image)
	work_layer = pdb.gimp_layer_new(new_image,width,height,RGBA_IMAGE,'circle',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,work_layer,None,0)

	active_vectors = pdb.gimp_image_get_active_vectors(image)
	num_strokes,stroke_ids = pdb.gimp_vectors_get_strokes(active_vectors)

	#create a new vectors on new image
	new_vectors = pdb.gimp_vectors_new(new_image,"rectangle to circle path")
	pdb.gimp_image_insert_vectors(new_image,new_vectors,None,0)
	for s in range(0,num_strokes):
		stype,num_points,control_points,closed = pdb.gimp_vectors_stroke_get_points(active_vectors,stroke_ids[s])
		
		#do transformation/calculations here
		new_control_points = [];
		for p in range(0,len(control_points)/2):
			x = control_points[p*2]
			y = control_points[p*2+1]
			#transform here to get points (xi,yi)
			angle = (float(x)/layer.width)*360.0
			radius = (layer.height-float(y))/layer.height*(outerR-innerR)+innerR
			xi = cx+math.cos(math.radians(angle)) * radius
			yi = cy+math.sin(math.radians(angle)) * radius
			#set it to the transformed points
			new_control_points.append(xi);
			new_control_points.append(yi);
		new_stroke = pdb.gimp_vectors_stroke_new_from_points(new_vectors,stype,num_points,new_control_points,closed)


	
	pdb.gimp_context_pop()
	#pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()

register(
	"python_fu_rectangle_to_circle_path",
	"Rectangle to circle",
	"Rectangle to circle",
	"Tin Tran",
	"Tin Tran",
	"September 25 2022",
	"<Image>/Python-Fu/Rectangle to Circle Path...",             #Menu path
	"RGB*, GRAY*",
	[
	#(PF_BRUSH, "brush", "Brush:", 0),
	(PF_SPINNER, "width", "Inner Radius:", 100, (10, 5000, 1)),
	(PF_SPINNER, "length", "Outer Radius:", 300, (10, 5000, 1)),
	#(PF_TOGGLE, "random_angle",   "Use random angle (when doing generally round blotches):", 0),
	#(PF_SPINNER, "px_height", "Height of each tile image:", 100, (1, 1000, 50)),
	],
	[],
	python_rectangle_to_circle_path)
main()

