#!/usr/bin/env python
# coding=utf-8
# rectangle_to_circle.py
# Created by Tin Tran https://gimplearn.net#
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html#
#
# ------------
#| Change Log |
# ------------
# 1. - Initial version
from gimpfu import *
import math
def python_rectangle_to_circle(image, layer,innerR,outerR):
	#pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	# pdb.script_fu_selection_to_pattern()
	width = int(outerR*2)
	height = int(outerR*2)
	cx = outerR
	cy = outerR
	new_image = pdb.gimp_image_new(width,height,RGB)
	pdb.gimp_display_new(new_image)
	work_layer = pdb.gimp_layer_new(new_image,width,height,RGBA_IMAGE,'circle',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,work_layer,None,0)
	angmultiplier = 1.0/360.0*layer.width
	topmultiplier = 1.0/(outerR-innerR)*layer.height
	for y in range(0,height):
		for x in range(0,width):
			dist = ((x-cx)**2 + (y-cy)**2)**0.5
			if (dist <= outerR and dist >= innerR): #it's on the circle do work.
				fromTop = (outerR - dist)*topmultiplier
				angle = math.degrees(math.atan2(cx-x,cy-y)) % 360
				fromLeft = layer.width-1-angle*angmultiplier
				try:
					num_channels,pixel = pdb.gimp_drawable_get_pixel(layer,fromLeft,fromTop)
					pdb.gimp_drawable_set_pixel(work_layer,x,y,num_channels,pixel)
				except:
					pass
	pdb.gimp_drawable_update(work_layer,0,0,width,height)					
	pdb.gimp_context_pop()
	#pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()

register(
	"python_fu_rectangle_to_circle",
	"Rectangle to circle",
	"Rectangle to circle",
	"Tin Tran",
	"Tin Tran",
	"September 25 2022",
	"<Image>/Python-Fu/Rectangle to Circle...",             #Menu path
	"RGB*, GRAY*",
	[
	#(PF_BRUSH, "brush", "Brush:", 0),
	(PF_SPINNER, "width", "Estimated Piece Width:", 100, (10, 5000, 1)),
	(PF_SPINNER, "length", "Estimated Piece Length:", 300, (10, 5000, 1)),
	#(PF_TOGGLE, "random_angle",   "Use random angle (when doing generally round blotches):", 0),
	#(PF_SPINNER, "px_height", "Height of each tile image:", 100, (1, 1000, 50)),
	],
	[],
	python_rectangle_to_circle)
main()
