#!/usr/bin/env python

# replace_colors_using_template.py Release 1
# Helps to solve this question on reddit: https://www.reddit.com/r/GIMP/comments/74g327/i_am_trying_to_change_entire_palettes_to_lessen/
# Created by Tin Tran
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release

#import math
#import string
#import Image
from gimpfu import *
#import re, tempfile
#import os
#from array import array
#get tempfile name here so that it only does it once instead of multiple times which clogs up document history.
#tempf=tempfile.NamedTemporaryFile(mode='w', suffix='.svg', dir=gimp.directory, delete=False)
def python_replace_colors_using_template(image, layer) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	template = pdb.gimp_image_get_layer_by_name(image,"template")
	
	#offsetx,offsety = pdb.gimp_drawable_offsets(template)
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	
	change_image = pdb.gimp_image_new(image.width,image.height,RGB)
	change_display = pdb.gimp_display_new(change_image)
	change_layer = pdb.gimp_layer_new_from_drawable(layer,change_image)
	pdb.gimp_image_insert_layer(change_image,change_layer,None,0)
	pdb.gimp_image_convert_indexed(change_image,NO_DITHER,MAKE_PALETTE,255,False,True,"")
	
	template_image = pdb.gimp_image_new(image.width,image.height,RGB)
	template_display = pdb.gimp_display_new(template_image)
	template_layer = pdb.gimp_layer_new_from_drawable(template,template_image)
	pdb.gimp_image_insert_layer(template_image,template_layer,None,0)
	pdb.gimp_image_convert_indexed(template_image,NO_DITHER,MAKE_PALETTE,255,False,True,"")
	
	result_image = pdb.gimp_image_duplicate(change_image)
	pdb.gimp_display_new(result_image)
	
	num_bytest,template_colors = pdb.gimp_image_get_colormap(template_image)
	num_bytesc,change_colors = pdb.gimp_image_get_colormap(change_image)
	num_bytesr,result_colors = pdb.gimp_image_get_colormap(result_image)
	final_colors = list(result_colors);
	pdb.gimp_image_convert_rgb(template_image)
	pdb.gimp_image_convert_rgb(change_image)
	for y in range(y1,y2):
		for x in range(x1,x2):
			num_channels, pixel = pdb.gimp_drawable_get_pixel(template_layer,x,y)
			template_color = (pixel[0],pixel[1],pixel[2])
			num_channels, pixel2 = pdb.gimp_drawable_get_pixel(change_layer,x,y)
			change_color = (pixel2[0],pixel2[1],pixel2[2])
			if (pixel[3] > 0): #it's not transparent.
				for t in range(0,int(num_bytest/3)):
					if (template_color == (template_colors[t*3],template_colors[t*3+1],template_colors[t*3+2])): #if we find template color
						for c in range(0,int(num_bytesc/3)):
							if (change_color == (change_colors[c*3],change_colors[c*3+1],change_colors[c*3+2])): # and we find chanage color
								final_colors[c*3] = template_colors[t*3]
								final_colors[c*3+1] = template_colors[t*3+1]
								final_colors[c*3+2] = template_colors[t*3+2]
						
	pdb.gimp_image_set_colormap(result_image,num_bytesr,final_colors)						
	
	#deletes images that aren't used anymore.
	pdb.gimp_display_delete(change_display)
	pdb.gimp_display_delete(template_display)

	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_replace_colors_using_template",
	"Replace Colors Using Template",
	"Replace Colors Using Template...",
	"Tin Tran",
	"Tin Tran",
	"Sept 2017",
	"<Image>/Python-Fu/Replace colors using template...",
	"*",      # Create a new image, don't work on an existing one
	[
	#(PF_OPTION, "arrow_side", "Arrows Ends:", SIDE_END, SIDE_NAMES),
	#(PF_TOGGLE, "arrow_close", "Arrows Close:", 0),
	#(PF_SPINNER, "stroke_line_width", "Stroke Line Width:", 6.0, (1, 500, 0.5)),
	#(PF_GRADIENT, "gradient", "Custom Gradient:", 0),
	#(PF_TOGGLE, "reverse_gradient", "Reverse Custom Gradient", 0),
	#(PF_TOGGLE, "semiflatten",   "Semiflatten (Replace partial transparency with the current background color):", 0),
	],
	[],
	python_replace_colors_using_template)

main()
