#!/usr/bin/env python

# run_code_on_visible_layers.py Rel 3
# Created by Tin Tran
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# 1. Turn visibility on for layers that you'd like to run the code on.
# 2. Determine what code you need to run. In a text editor, type out your code (python) 
#    (refer to Help > Procedure Browser). 
#  a) Choose a function that probably starts with "python-fu" or "script-fu". 
#     Change all the hyphens (-) in the function name to underscores (_).
#  b) Add "pdb." in front of your function name.
#  c) Add parameters after your function. It probably needs an image and a layer 
#     as parameters before other parameters, so use "image" and "layer".
#  d) for example: pdb.script_fu_drop_shadow(image,layer,4,4,15,(0,0,0),60,False)
# 3. Run script (Python-fu/Run Code on Visible Layers...). 
#    Paste your line of code edited from step 2 into the "Code to Run:" option. Click "OK".
# 4. The Script will try to loop through all your visible layers pointing "layer" 
#    to the looped layers, and apply your code to each layer.
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
# Rel 2: Set active layer as well so that gmic filter can work on active layer.
# Rel 3: Layer groups options
from gimpfu import *
import math
#from inspect import formatargspec, getfullargspec

def python_run_code_on_visible_layers(image, layer, layergroup, code) : #FUNCTION DEFINITION
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#YOUR CODE BEGINS=======================
	apply_layers = []
	for layer in image.layers:
		if layergroup == 0: #All layers
			apply_layers.append(layer)
		elif layergroup == 1: #Visible layers
			if layer.visible:
				apply_layers.append(layer)
		elif layergroup == 2: #Not visible layers
			if not layer.visible:
				apply_layers.append(layer)
		elif layergroup == 3: #Linked layers
			if layer.linked:
				apply_layers.append(layer)
		elif layergroup == 4: #Not-linked layers
			if not layer.linked:
				apply_layers.append(layer)
				
	#apply code to apply_layers now
	for layer in apply_layers:
		pdb.gimp_image_set_active_layer(image,layer)
		exec(code)
	#YOUR CODE ENDS ========================
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_run_code_on_visible_layers",
	"Run code on selected layers",
	"Run code on selected layers...",
	"Tin Tran",
	"Tin Tran",
	"Oct 2017",
	"<Image>/Python-Fu/Run Code on Visible Layers...",
	"*",      # Create a new image, don't work on an existing one
	[ 
	#INPUT BEGINS
	(PF_OPTION, "layergroup","Choose on what layers", 1, ["ALL LAYERS","VISIBLE LAYERS","NOT-VISIBLE LAYERS","LINKED LAYERS","NOT-LINKED LAYERS"]),
	(PF_STRING, "code", "Code to Run:", "pdb.script_fu_drop_shadow(image,layer,4,4,15,(0,0,0),60,False)"),
	#(PF_OPTION, "arrow_side", "Arrows Ends:", SIDE_END, SIDE_NAMES),
	#(PF_TOGGLE, "arrow_close", "Arrows Close:", 0),
	#(PF_SPINNER, "border_width", "Border Width (bevelled width):", 10, (0, 500, 1)),
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_blur_radius", "Shadow Blur Radius:", 15, (0,1024,1)),
	#(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 100, (0,100,1)),
	#INPUT ENDS
	],
	[],
	python_run_code_on_visible_layers)

main()
