#!/usr/bin/env python

# slider_animation.py Rel 2
# Created by Tin Tran
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
# Rel 2: Added "Circle" option.
from gimpfu import *

def python_slider_animation(image, layer, slider_color, slider_width, frames_per_slide, slider_style) : #FUNCTION DEFINITION
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#YOUR CODE BEGINS=======================
	new_image = pdb.gimp_image_duplicate(image)
	pdb.gimp_display_new(new_image)
	layers_count = len(new_image.layers)
	#get animate all layers except bottom most layer.
	animate_layers = list(new_image.layers)
	#add alpha if it doesn't have it.
	for layer in animate_layers:
		if not layer.has_alpha:
			pdb.gimp_layer_add_alpha(layer)
	
	bottom_layer = pdb.gimp_layer_new_from_drawable(new_image.layers[0],new_image)
	pdb.gimp_image_insert_layer(new_image,bottom_layer,None,layers_count)
	if slider_style == 0:
		slider_layer = pdb.gimp_layer_new(new_image,slider_width,image.height,RGBA_IMAGE,"slider",100,NORMAL_MODE)
	elif slider_style  == 1: #circle
		slider_layer = pdb.gimp_layer_new(new_image,image.width,image.height,RGBA_IMAGE,"slider",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(new_image,slider_layer,None,0)
	pdb.gimp_context_set_foreground(slider_color)
	pdb.gimp_edit_fill(slider_layer,FOREGROUND_FILL)
	
	if slider_style  == 0:
		start = -slider_width
		inc = float(image.width+slider_width)/frames_per_slide
		for i in range(0,len(animate_layers)):
			current_frame = animate_layers[i]
			for x in range(0,int(frames_per_slide+1)):
				pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,0,0,min(image.width,max(0,x*inc)),image.height)
				non_empty,x1,y2,x2,y2 = pdb.gimp_selection_bounds(new_image)
				if non_empty:
					pdb.gimp_edit_clear(current_frame)
				pdb.gimp_layer_set_offsets(slider_layer,start + x*inc,0)
				frame_layer = pdb.gimp_layer_new_from_visible(new_image,new_image,"frame")
				pdb.gimp_image_insert_layer(new_image,frame_layer,None,0)
				frame_layer.visible = False
	elif slider_style  == 1:
		radius = ((image.width/2)**2 + (image.height/2)**2)**0.5
		start = -slider_width
		inc = float(radius+slider_width)/frames_per_slide
		for i in range(0,len(animate_layers)):
			current_frame = animate_layers[i]
			for x in range(0,int(frames_per_slide+1)):
				pdb.gimp_selection_all(new_image)
				pdb.gimp_edit_clear(slider_layer)
				pdb.gimp_image_select_ellipse(new_image,CHANNEL_OP_REPLACE,image.width/2-max(0,start+inc*x+slider_width),image.height/2-max(0,start+inc*x+slider_width),max(0,start+inc*x+slider_width)*2,max(0,start+inc*x+slider_width)*2)
				pdb.gimp_image_select_ellipse(new_image,CHANNEL_OP_SUBTRACT,image.width/2-max(0,start+inc*x),image.height/2-max(0,start+inc*x),max(0,start+inc*x)*2,max(0,start+inc*x)*2)
				non_empty,x1,y2,x2,y2 = pdb.gimp_selection_bounds(new_image)
				if non_empty:
					pdb.gimp_edit_fill(slider_layer,FOREGROUND_FILL)
				pdb.gimp_image_select_ellipse(new_image,CHANNEL_OP_REPLACE,image.width/2-max(0,start+inc*x),image.height/2-max(0,start+inc*x),max(0,start+inc*x)*2,max(0,start+inc*x)*2)
				non_empty,x1,y2,x2,y2 = pdb.gimp_selection_bounds(new_image)
				if non_empty:
					pdb.gimp_edit_clear(current_frame)
				#pdb.gimp_layer_set_offsets(slider_layer,start + x*inc,0)
				frame_layer = pdb.gimp_layer_new_from_visible(new_image,new_image,"frame")
				pdb.gimp_image_insert_layer(new_image,frame_layer,None,0)
				frame_layer.visible = False
	#delete no longer needed layers
	pdb.gimp_image_remove_layer(new_image,bottom_layer)
	pdb.gimp_image_remove_layer(new_image,slider_layer)
	for layer in animate_layers:
		pdb.gimp_image_remove_layer(new_image,layer)
	
	for layer in new_image.layers:
		layer.visible = True
	
	
	#YOUR CODE ENDS ========================
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_slider_animation",
	"Slider animation",
	"Slider animation...",
	"Tin Tran",
	"Tin Tran",
	"Oct 2017",
	"<Image>/Python-Fu/Slider Animation...",
	"*",      # Create a new image, don't work on an existing one
	[ 
	#INPUT BEGINS
	(PF_COLOR, "slider_color", "Slider Color:", (0, 0, 0) ), # extra param is RGB triple
	(PF_SPINNER, "slider_width", "Slider Width:", 10, (0, 200, 1)),
	(PF_SPINNER, "frames_per_slide", "Frames Per Slide:", 20, (5, 200, 1)),
	(PF_OPTION, "slider_style", "Slider Style:", 0, ["Bar", "Circle"]),
	
	#(PF_GRADIENT, "alphaLayersGradientA", "AlphaLayersGradientA (by PegLeg44):", "AlphaLayersGradientA"),
	#(PF_GRADIENT, "alphaLayersGradientB", "AlphaLayersGradientB (by Pegleg44):", "AlphaLayersGradientB"),
	#(PF_TOGGLE, "optionalGradientMap", "Optional Gradient Map:", 0),
	#(PF_GRADIENT, "optionalGradient", "Optional Gradient:",0),
	#(PF_STRING, "code", "Code to Run:", "pdb.script_fu_drop_shadow(image,layer,4,4,15,(0,0,0),60,False)"),
	#(PF_OPTION, "arrow_side", "Arrows Ends:", SIDE_END, SIDE_NAMES),
	#(PF_TOGGLE, "arrow_close", "Arrows Close:", 0),
	#(PF_SPINNER, "border_width", "Border Width (bevelled width):", 10, (0, 500, 1)),
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_blur_radius", "Shadow Blur Radius:", 15, (0,1024,1)),
	#(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 100, (0,100,1)),
	#INPUT ENDS
	],
	[],
	python_slider_animation)

main()
