#!/usr/bin/env python

# Stroke_Path_With_Gradient.py Release 2
# To stroke multiple paths with gradient, intended to be used with ofnuts path-inbetweener-0.2.py found here: https://sourceforge.net/projects/gimp-path-tools/files/scripts/
# Created by Tin Tran
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
# Rel 2: Fixed error so it would work on Linux
import math
import string
#import Image
from gimpfu import *
import re, tempfile
import os
#from array import array
#get tempfile name here so that it only does it once instead of multiple times which clogs up document history.
#tempf=tempfile.NamedTemporaryFile(mode='w', suffix='.svg', dir=gimp.directory, delete=False)
tempf=tempfile.NamedTemporaryFile(mode='w', suffix='.svg', dir=gimp.directory, delete=False)
tempf.write("whatever")
tempf.close()
def color_to_hex(color):
	whole_color = (color[0],color[1],color[2]);
	return '#%02x%02x%02x' % whole_color
def vector_to_line_stroke(image, vector, layer, color="#000000", width=1, capstyle="butt", joinstyle="miter", miterlimit=10, shaperendering="auto",first=1):
	newelements = {
			'stroke': color,
			'stroke-width': width,
			'stroke-linecap': capstyle,
			'stroke-linejoin': joinstyle,
			'stroke-miterlimit': miterlimit,
			'shape-rendering': shaperendering,
			}
	svg = pdb.gimp_vectors_export_to_string(image, vector)
    #fix width and height to be resolution (px/inch)-independent
	svg = re.sub(r'(<svg\s[^>]*\swidth\s*=\s*)\S*"', r'\1"%dpx"' % image.width, svg, flags=re.DOTALL)
	svg = re.sub(r'(<svg\s[^>]*\sheight\s*=\s*)\S*"', r'\1"%dpx"' % image.height, svg, flags=re.DOTALL)
	svg = re.sub(r'(<path\s[^>]*)\sstroke\s*=\s*"black"', r'\1', svg, flags=re.DOTALL)
	svg = re.sub(r'(<path\s[^>]*)\sstroke-width\s*=\s*"1"', r'\1', svg, flags=re.DOTALL)
	svg = re.sub(r'(<path\s)', r'\1' + ''.join([r'%s="%s" ' % i for i in newelements.items()]), svg, flags=re.DOTALL)
	if (first == 1):
		#do nothing
		pass
	else:
		#remove xml tag, doctype tag, svg tag so that it's left with just path tag.
		svg = re.sub(r'(<\?xml\s[^>]*>)(.*)(<\!DOCTYPE\s[^>]*>)(.*)(<svg\s[^>]*>)(.*)(<path\s[^>]*>)(.*)(<\/svg>)', r'\7', svg, flags=re.DOTALL)
		#pdb.gimp_message(svg);
		
	return svg;
#custom function to append the multiple paths elements into existing svg xml.
def svg_append(svg,svgpaths):
	svg = re.sub(r'(<\?xml\s[^>]*>)(.*)(<\!DOCTYPE\s[^>]*>)(.*)(<svg\s[^>]*>)(.*)(<path\s[^>]*>)(.*)(<\/svg>)', r'\1\2\3\4\5\6\7\8', svg, flags=re.DOTALL)
	svg += svgpaths;
	svg += "</svg>";
	return svg;
#write to svg file and load as layer in GIMP
def write_to_file_and_load(image,layer,svg):
	tf = open(tempf.name,"w")
	tf.write(svg)
	tf.close()
	newlayer = pdb.gimp_file_load_layer(image, tf.name)
	#os.remove(tf.name) Try not removing it so that user can open it as svg
	image.add_layer(newlayer) #needs to be added to the image to be able to copy from
	copylayer = pdb.gimp_edit_named_copy(newlayer, "stroke")
	image.remove_layer(newlayer)
	floating_sel = pdb.gimp_edit_named_paste(layer, copylayer, True)
	pdb.gimp_floating_sel_anchor(floating_sel)
#=====================================================================================================================
def python_stroke_paths_with_gradient(image, layer,stroke_line_width,gradient,reverse_gradient,semiflatten) :
	pdb.gimp_image_undo_group_start(image)
	
	#grab selection bounding box values
	
	#get active vectors and do some work
	visible_vectors = []
	for i in range(0, len(image.vectors)):
		if image.vectors[i].visible == 1:
			visible_vectors.append(image.vectors[i])
			
	visible_vectors = visible_vectors[::-1] #reverse order so that top vectors are drawn last.
	#active_vectors = pdb.gimp_image_get_active_vectors(image)
	#num_strokes,stroke_ids = pdb.gimp_vectors_get_strokes(active_vectors)
	
	layer_name = "Gradient Stroked Paths";
	width = pdb.gimp_image_width(image)
	height = pdb.gimp_image_height(image)
	gradient_layer = pdb.gimp_layer_new(image,width,height,RGBA_IMAGE,layer_name,100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,gradient_layer,None,-1)
	layer_name = pdb.gimp_item_get_name(gradient_layer)
	svgpaths = "";
	#grab colors samples if using custom gradient.
	strokes = len(visible_vectors)
	num_color_samples, colors = pdb.gimp_gradient_get_uniform_samples(gradient,strokes,reverse_gradient);
	for stroke in range(0,strokes):
		color = (colors[stroke * 4 + 0]*255,colors[stroke * 4 + 1]*255, colors[stroke * 4 + 2]*255);
			
		stroke_width = stroke_line_width # - ((stroke_line_width-0.01)/strokes * stroke);
		if stroke==0:
			svg = vector_to_line_stroke(image,visible_vectors[stroke],gradient_layer,color_to_hex(color),stroke_width,"square","miter",10,"auto",stroke==0)
		else:
			svgpaths += vector_to_line_stroke(image,visible_vectors[stroke],gradient_layer,color_to_hex(color),stroke_width,"square","miter",10,"auto",stroke==0)
		#after it's done we just append svgpaths to svg
		svg = svg_append(svg,svgpaths)
		write_to_file_and_load(image,gradient_layer,svg)
		
	if semiflatten == 1:
		pdb.plug_in_semiflatten(image,gradient_layer)
	
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_stroke_paths_with_gradient",
	"Stroke Paths with Gradient",
	"Stroke Paths with Gradient...",
	"Tin Tran",
	"Tin Tran",
	"Sept 2017",
	"<Image>/Python-Fu/Stroke Paths With Gradient...",
	"*",      # Create a new image, don't work on an existing one
	[
	#(PF_OPTION, "arrow_side", "Arrows Ends:", SIDE_END, SIDE_NAMES),
	#(PF_TOGGLE, "arrow_close", "Arrows Close:", 0),
	(PF_SPINNER, "stroke_line_width", "Stroke Line Width:", 6.0, (1, 500, 0.5)),
	(PF_GRADIENT, "gradient", "Custom Gradient:", 0),
	(PF_TOGGLE, "reverse_gradient", "Reverse Custom Gradient", 0),
	(PF_TOGGLE, "semiflatten",   "Semiflatten (Replace partial transparency with the current background color):", 0),
	],
	[],
	python_stroke_paths_with_gradient)

main()
