#!/usr/bin/env python

# tile_selection.py Release 1
# Repeat selection horizontall/vertically.
# Created by Tin Tran http://gimplearn.net
# Comments directed to http://gimplearn.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
#import random
#import string
#import Image
from gimpfu import *
#from array import array
#import sys

def python_tt_tile_selection(image, layer, repeat, repeatXY, offsetXY):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()

	#turn selection into tile
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	if non_empty:
		tile_width = x2 - x1
		tile_height = y2 - y1
		tile = pdb.gimp_layer_new(image,tile_width,tile_height,RGBA_IMAGE,"Tile",100,LAYER_MODE_NORMAL)
		pdb.gimp_image_insert_layer(image,tile,None,0)
		pdb.gimp_layer_set_offsets(tile,x1,y1)
		pdb.gimp_context_set_foreground((0,0,0)) #make foreground pureblack to fill tile with
		#use this instead of other methods for speed in GIMP 2.10
		pdb.gimp_edit_blend(tile,BLEND_FG_BG_RGB,LAYER_MODE_NORMAL,GRADIENT_LINEAR,100,0,REPEAT_NONE,FALSE,FALSE,3,0.2,FALSE,x1,y1,x2,y2)
		#now we have the tile.
		if repeat == 0: #Vertically
			switchX = 0
			switchY = 1
		elif repeat == 1: #Horizontally
			switchX = 1
			switchY = 0
		incX = repeatXY * switchX
		incY = repeatXY * switchY
		offsetX = offsetXY * switchY
		offsetY = offsetXY * switchX

		#select in negative direction
		x,y = x1,y1
		while (y > 0-tile_height) and (x > 0-tile_width):
			x -= incX
			y -= incY
			x -= offsetX
			y -= offsetY
			pdb.gimp_layer_set_offsets(tile,x,y)
			pdb.gimp_image_select_item(image,CHANNEL_OP_ADD,tile)
		#select in positive direction	
		x,y = x1,y1
		while (y < image.height) and (x < image.width):
			x += incX
			y += incY
			x += offsetX
			y += offsetY
			pdb.gimp_layer_set_offsets(tile,x,y)
			pdb.gimp_image_select_item(image,CHANNEL_OP_ADD,tile)
		pdb.gimp_image_remove_layer(image,tile)
	else:
		pdb.gimp_message("No selection detected. Please make a selection (tile selection) and re-run plug-in!")	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
register(
	"python_fu_tt_tile_selection",                           
	"Select a Tile repeatedly through out layer",
	"Select a Tile repeatedly through out layer",
	"Tin Tran",
	"Tin Tran",
	"June 2019",
	"<Image>/Python-Fu/Tile Selection...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_OPTION, "repeat", "Repeat Selection Type", 1, ["Repeat Vertically", "Repeat Horizontally"]),
	(PF_SPINNER, "repeatXY", "Repeat every (pixels):", 46, (1, 5000, 1)),
	(PF_SPINNER, "offsetXY", "Offset X/Y for Repeat Vertically/Horizontally):", 0, (-5000, 5000, 1)),
	#(PF_TOGGLE, "brushit", "Edit-Blend instead of Edit-Fill (Edit-blend is faster):", 1),
	#(PF_SPINNER, "brushsize", "Brush Size:", 500, (100, 10000, 1)),
	#(PF_TOGGLE, "dodge", "Dodge filled areas (Subtract alpha to selection's selection):", 0),
	],
	[],
	python_tt_tile_selection)

main()